@@ -52,9 +52,7 @@ from pygpsclient._version import __version__ as VERSION
 from pygpsclient.dialog_state import dialog_state
 from pygpsclient.file_handler import FileHandler
 from pygpsclient.globals import (
-    BADCOL,
     CFG,
-    CHECK_FOR_UPDATES,
     CLASS,
     CONFIGFILE,
     CONNECTED,
@@ -65,6 +63,7 @@ from pygpsclient.globals import (
     DEFAULT_REGION,
     DEFAULT_USER,
     DISCONNECTED,
+    ERRCOL,
     FRAME,
     GNSS_EOF_EVENT,
     GNSS_ERR_EVENT,
@@ -72,11 +71,14 @@ from pygpsclient.globals import (
     GNSS_TIMEOUT_EVENT,
     GUI_UPDATE_INTERVAL,
     ICON_APP128,
+    INFOCOL,
     MQTT_PROTOCOL,
     NOPORTS,
     NTRIP_EVENT,
     OKCOL,
     SOCKSERVER_MAX_CLIENTS,
+    SPARTN_BASEDATE_CURRENT,
+    SPARTN_DEFAULT_KEY,
     SPARTN_EVENT,
     SPARTN_OUTPORT,
     SPARTN_PPSERVER_URL,
@@ -109,19 +111,20 @@ from pygpsclient.strings import (
 )
 from pygpsclient.ubx_handler import UBXHandler
 from pygpsclient.widget_state import (
+    COL,
     COLSPAN,
     DEFAULT,
     HIDE,
     MAXCOLSPAN,
     MAXROWSPAN,
     MENU,
+    ROW,
     ROWSPAN,
     SHOW,
     STICKY,
     VISIBLE,
-    WDGBANNER,
-    WDGSETTINGS,
-    WDGSTATUS,
+    WDGCHART,
+    WDGCONSOLE,
     widget_state,
 )
 
@@ -159,7 +162,8 @@ class App(Frame):
         mqttclientmode = int(
             kwargs.pop("mqttclientmode", getenv("MQTTCLIENTMODE", "0"))
         )
-        spartnkey = kwargs.pop("spartnkey", getenv("MQTTKEY", ""))
+        spartnkey = kwargs.pop("spartnkey", getenv("MQTTKEY", SPARTN_DEFAULT_KEY))
+        spartnbasedate = kwargs.pop("spartnbasedate", SPARTN_BASEDATE_CURRENT)
         ntripcaster_user = kwargs.pop(
             "ntripcasteruser", getenv("NTRIPCASTER_USER", DEFAULT_USER)
         )
@@ -201,6 +205,7 @@ class App(Frame):
         self._socket_server = None
         self._colcount = 0
         self._rowcount = 0
+        self._consoledata = []
 
         # Load configuration from file if it exists
         self._colortags = []
@@ -221,8 +226,10 @@ class App(Frame):
             self.saved_config["mqapikey_s"] = mqapikey
         if mqttclientid != "":
             self.saved_config["mqttclientid_s"] = mqttclientid
-        if spartnkey != "":
+        if spartnkey != SPARTN_DEFAULT_KEY:
             self.saved_config["spartnkey_s"] = spartnkey
+        if spartnbasedate != SPARTN_BASEDATE_CURRENT:
+            self.saved_config["spartnbasedate_n"] = spartnkey
         if mqttclientregion != DEFAULT_REGION:
             self.saved_config["mqttclientregion_s"] = mqttclientregion
         if mqttclientmode != 0:
@@ -243,15 +250,15 @@ class App(Frame):
         # display config load status once status frame has been instantiated
         if configerr == "":
             if self._nowidgets:  # if all widgets have been disabled in config
-                self.set_status(NOWDGSWARN.format(self._configfile), BADCOL)
+                self.set_status(NOWDGSWARN.format(self._configfile), ERRCOL)
             else:
                 self.set_status(LOADCONFIGOK.format(self._configfile), OKCOL)
         else:
             if "No such file or directory" in configerr:
-                self.set_status(LOADCONFIGNONE.format(self._configfile), BADCOL)
+                self.set_status(LOADCONFIGNONE.format(self._configfile), ERRCOL)
             else:
                 self.set_status(
-                    LOADCONFIGBAD.format(self._configfile, configerr), BADCOL
+                    LOADCONFIGBAD.format(self._configfile, configerr), ERRCOL
                 )
 
         # initialise widgets
@@ -262,10 +269,10 @@ class App(Frame):
 
         self.frm_banner.update_conn_status(DISCONNECTED)
         if self.frm_settings.frm_serial.status == NOPORTS:
-            self.set_status(INTROTXTNOPORTS, BADCOL)
+            self.set_status(INTROTXTNOPORTS, ERRCOL)
 
         # Check for more recent version (if enabled)
-        if CHECK_FOR_UPDATES:
+        if self.saved_config.get("checkforupdate_b", False):
             self._check_update()
 
     def _body(self):
@@ -277,10 +284,12 @@ class App(Frame):
         self.menu = MenuBar(self)
         self.__master.config(menu=self.menu)
 
-        # dynamically instantiate widgets defined in widgets_grid
+        # instantiate widgets
         for value in widget_state.values():
-            _ = setattr(
-                self, value[FRAME], value[CLASS](self, borderwidth=2, relief="groove")
+            setattr(
+                self,
+                value[FRAME],
+                value[CLASS](self, borderwidth=2, relief="groove"),
             )
 
     def _do_layout(self):
@@ -289,97 +298,124 @@ class App(Frame):
         widget visibility and menu label (show/hide).
         """
 
-        col = mcol = 0
-        row = mrow = 1
-        for i, nam in enumerate(widget_state):
-            if i > 2:  # only position dynamic widgets
-                col, row = self._grid_widget(nam, col, row)
-            mcol = max(col, mcol)
-            mrow = max(row, mrow)
+        col = 0
+        row = 1
+        maxcol = 0
+        maxrow = 0
+        men = 0
+        for name in widget_state:
+            col, row, maxcol, maxrow, men = self._widget_grid(
+                name, col, row, maxcol, maxrow, men
+            )
 
+        # ensure widgets expand to size of container (needed
+        # when not using 'pack' grid management)
+        # weight = 0 means fixed, non-expandable
+        # weight > 0 means expandable
         for col in range(MAXCOLSPAN + 1):
-            self.__master.grid_columnconfigure(col, weight=0 if col > mcol - 1 else 1)
-        for row in range(1, MAXROWSPAN + 2):
-            self.__master.grid_rowconfigure(row, weight=0 if row > mrow else 1)
+            self.__master.grid_columnconfigure(col, weight=0)
+        for row in range(MAXROWSPAN + 2):
+            self.__master.grid_rowconfigure(row, weight=0)
+        # print(f"{maxcol=} {maxrow=}")
+        for col in range(maxcol):
+            self.__master.grid_columnconfigure(col, weight=5)
+        for row in range(1, maxrow + 1):
+            self.__master.grid_rowconfigure(row, weight=5)
 
-        self._grid_widget(WDGSETTINGS, mcol, 1, 1, mrow)  # always on top
-        self._grid_widget(WDGBANNER, 0, 0, mcol + 1, 1)  # always on right
-        self._grid_widget(WDGSTATUS, 0, mrow + 1, mcol + 1, 1)  # always on bottom
-
-    def _grid_widget(
-        self, nam: str, col: int, row: int, colspan: int = 1, rowspan: int = 1
+    def _widget_grid(
+        self, name: str, col: int, row: int, maxcol: int, maxrow: int, men: int
     ) -> tuple:
         """
-        Arrange individual widget and update menu label (show/hide).
+        Arrange widgets and update menu label (show/hide).
 
-        :param str nam: name of widget
-        :param int col: column
+        Widgets with explicit COL settings will be placed in fixed
+        positions; widgets with no COL setting will be arranged
+        dynamically.
+
+        :param str name: name of widget
+        :param int col: col
         :param int row: row
-        :param int colspan: optional columnspan
-        :param int rowspan: optional rowspan
-        :return: next available (col, row)
+        :param int maxcol: max cols
+        :param int maxrow: max rows
+        :param int men: menu position
+        :return: max row & col
         :rtype: tuple
         """
 
-        wdg = widget_state[nam]
+        wdg = widget_state[name]
+        dynamic = wdg.get(COL, None) is None
         frm = getattr(self, wdg[FRAME])
         if wdg[VISIBLE]:
-            colspan = wdg.get(COLSPAN, colspan)
-            rowspan = wdg.get(ROWSPAN, rowspan)
-            if col >= MAXCOLSPAN and nam != WDGSETTINGS:
-                col = 0
-                row += rowspan
-            # keep track of cumulative cols & rows
-            ccol = wdg.get("col", col)
-            crow = wdg.get("row", row)
+            self.widget_enable_messages(name)
+            fcol = wdg.get(COL, col)
+            frow = wdg.get(ROW, row)
+            colspan = wdg.get(COLSPAN, 1)
+            rowspan = wdg.get(ROWSPAN, 1)
+            if dynamic and fcol + colspan > MAXCOLSPAN:
+                fcol = 0
+                frow += 1
             frm.grid(
-                column=ccol,
-                row=crow,
+                column=fcol,
+                row=frow,
                 columnspan=colspan,
                 rowspan=rowspan,
                 padx=2,
                 pady=2,
                 sticky=wdg.get(STICKY, (N, S, W, E)),
             )
-            col += colspan
             lbl = HIDE
+            if dynamic:
+                col += colspan
+                if col >= MAXCOLSPAN:
+                    col = 0
+                    row += rowspan
+                maxcol = max(maxcol, fcol + colspan)
+                maxrow = max(maxrow, frow)
         else:
             frm.grid_forget()
             lbl = SHOW
 
         # update menu label (show/hide)
-        if wdg[MENU] is not None:
-            self.menu.view_menu.entryconfig(wdg[MENU], label=f"{lbl} {nam}")
+        if wdg.get(MENU, True):
+            self.menu.view_menu.entryconfig(men, label=f"{lbl} {name}")
+            men += 1
 
         # force widget to rescale
         frm.event_generate("<Configure>")
 
-        return col, row
+        return col, row, maxcol, maxrow, men
 
-    def toggle_widget(self, widget: str):
+    def widget_toggle(self, name: str):
         """
         Toggle widget visibility and enable or disable any
         UBX messages required by widget.
 
-        :param str widget: widget name
+        :param str name: widget name
         """
 
-        wdg = widget_state[widget]
+        wdg = widget_state[name]
         wdg[VISIBLE] = not wdg[VISIBLE]
+        self._do_layout()
 
+    def widget_enable_messages(self, name: str):
+        """
+        Enable any NMEA, UBX or RTCM messages required by widget.
+
+        :param str name: widget name
+        """
+
+        wdg = widget_state[name]
         frm = getattr(self, wdg[FRAME])
         if hasattr(frm, "enable_messages"):
             frm.enable_messages(wdg[VISIBLE])
 
-        self._do_layout()
-
-    def reset_widgets(self):
+    def widget_reset(self):
         """
         Reset widgets to default layout.
         """
 
         for _, wdg in widget_state.items():
-            wdg[VISIBLE] = wdg[DEFAULT]
+            wdg[VISIBLE] = wdg.get(DEFAULT, False)
         self._do_layout()
 
     def reset_gnssstatus(self):
@@ -434,6 +470,7 @@ class App(Frame):
 
         """
 
+        color = INFOCOL if color == "blue" else color
         self.frm_status.set_status(message, color)
 
     def set_event(self, evt: str):
@@ -454,7 +491,7 @@ class App(Frame):
         if self.conn_status == DISCONNECTED and self.rtk_conn_status == DISCONNECTED:
             self.set_status("", OKCOL)
         else:
-            self.set_status(DLGSTOPRTK, BADCOL)
+            self.set_status(DLGSTOPRTK, ERRCOL)
             return
 
         (filename, config, configerr) = self.file_handler.load_config(None)
@@ -472,13 +509,13 @@ class App(Frame):
                 frm.reset()
             self._do_layout()
             if self._nowidgets:
-                self.set_status(NOWDGSWARN.format(filename), BADCOL)
+                self.set_status(NOWDGSWARN.format(filename), ERRCOL)
             else:
                 self.set_status(LOADCONFIGOK.format(filename), OKCOL)
         elif configerr == "cancelled":  # user cancelled
             return
         else:  # config error
-            self.set_status(LOADCONFIGBAD.format(filename), BADCOL)
+            self.set_status(LOADCONFIGBAD.format(filename), ERRCOL)
 
     def save_config(self):
         """
@@ -497,7 +534,7 @@ class App(Frame):
         if err == "":
             self.set_status(SAVECONFIGOK, OKCOL)
         else:  # save failed
-            self.set_status(SAVECONFIGBAD.format(err), BADCOL)
+            self.set_status(SAVECONFIGBAD.format(err), ERRCOL)
 
     def update_widgets(self):
         """
@@ -509,6 +546,11 @@ class App(Frame):
 
         try:
             self._nowidgets = True
+            for key, vals in widget_state.items():
+                vis = self.saved_config.get(key, False)
+                vals[VISIBLE] = vis
+                if vis:
+                    self._nowidgets = False
             for key, vals in widget_state.items():
                 vis = self.saved_config.get(key, False)
                 vals[VISIBLE] = vis
@@ -517,7 +559,7 @@ class App(Frame):
             if self._nowidgets:
                 widget_state["Status"][VISIBLE] = "true"
         except KeyError as err:
-            self.set_status(f"{CONFIGERR} - {err}", BADCOL)
+            self.set_status(f"{CONFIGERR} - {err}", ERRCOL)
 
     def update_NTRIP_handler(self):
         """
@@ -572,12 +614,12 @@ class App(Frame):
                 ntripsettings["spartndecode"] = 0
             # if basedate is provided in config file, it must be an integer gnssTimetag
             ntripsettings["spartnbasedate"] = self.saved_config.get(
-                "spartnbasedate_i", date2timetag(datetime.now(timezone.utc))
+                "spartnbasedate_n", date2timetag(datetime.now(timezone.utc))
             )
             self.ntrip_handler.settings = ntripsettings
 
         except (KeyError, ValueError, TypeError, TclError) as err:
-            self.set_status(f"Error processing config data: {err}", BADCOL)
+            self.set_status(f"Error processing config data: {err}", ERRCOL)
 
     def update_SPARTN_handler(self):
         """
@@ -615,18 +657,22 @@ class App(Frame):
             spartnsettings["output"] = self.spartn_inqueue
             spartnsettings["spartndecode"] = self.saved_config.get("spartndecode_b", 0)
             spartnsettings["spartnkey"] = self.saved_config.get(
-                "spartnkey_s", getenv("MQTTKEY", "")
+                "spartnkey_s", getenv("MQTTKEY", SPARTN_DEFAULT_KEY)
             )
             if spartnsettings["spartnkey"] == "":
                 spartnsettings["spartndecode"] = 0
             # if basedate is provided in config file, it must be an integer gnssTimetag
-            spartnsettings["spartnbasedate"] = self.saved_config.get(
-                "spartnbasedate_i", date2timetag(datetime.now(timezone.utc))
+            basedate = self.saved_config.get(
+                "spartnbasedate_n", SPARTN_BASEDATE_CURRENT
             )
+            if basedate == SPARTN_BASEDATE_CURRENT:
+                basedate = date2timetag(datetime.now(timezone.utc))
+            spartnsettings["spartnbasedate"] = basedate
             self.spartn_handler.settings = spartnsettings
+            self.logger.debug(f"{self.spartn_handler.settings=}")
 
         except (KeyError, ValueError, TypeError, TclError) as err:
-            self.set_status(f"Error processing config data: {err}", BADCOL)
+            self.set_status(f"Error processing config data: {err}", ERRCOL)
 
     def start_dialog(self, dlg: str):
         """
@@ -745,7 +791,7 @@ class App(Frame):
             ) as self._socket_server:
                 self._socket_server.serve_forever()
         except OSError as err:
-            self.set_status(f"Error starting socket server {err}", BADCOL)
+            self.set_status(f"Error starting socket server {err}", ERRCOL)
 
     def update_clients(self, clients: int):
         """
@@ -797,8 +843,9 @@ class App(Frame):
         self.frm_settings.frm_socketserver.socketserving = (
             False  # turn off socket server
         )
+        self._refresh_widgets()
         self.conn_status = DISCONNECTED
-        self.set_status(ENDOFFILE, "red")
+        self.set_status(ENDOFFILE, ERRCOL)
 
     def on_gnss_timeout(self, event):  # pylint: disable=unused-argument
         """
@@ -811,8 +858,20 @@ class App(Frame):
         self.frm_settings.frm_socketserver.socketserving = (
             False  # turn off socket server
         )
+        self._refresh_widgets()
+        self.conn_status = DISCONNECTED
+        self.set_status(INACTIVE_TIMEOUT, ERRCOL)
+
+    def on_stream_error(self, event):  # pylint: disable=unused-argument
+        """
+        EVENT TRIGGERED
+        Action on "<<gnss_error>>" event - connection streaming error.
+
+        :param event event: <<gnss_error>> event
+        """
+
+        self._refresh_widgets()
         self.conn_status = DISCONNECTED
-        self.set_status(INACTIVE_TIMEOUT, "red")
 
     def on_ntrip_read(self, event):  # pylint: disable=unused-argument
         """
@@ -844,7 +903,7 @@ class App(Frame):
         except Empty:
             pass
         except (SerialException, SerialTimeoutException) as err:
-            self.set_status(f"Error sending to device {err}", BADCOL)
+            self.set_status(f"Error sending to device {err}", ERRCOL)
 
     def on_spartn_read(self, event):  # pylint: disable=unused-argument
         """
@@ -875,17 +934,7 @@ class App(Frame):
         except Empty:
             pass
         except (SerialException, SerialTimeoutException) as err:
-            self.set_status(f"Error sending to device {err}", BADCOL)
-
-    def on_stream_error(self, event):  # pylint: disable=unused-argument
-        """
-        EVENT TRIGGERED
-        Action on "<<gnss_error>>" event - connection streaming error.
-
-        :param event event: <<gnss_error>> event
-        """
-
-        self.conn_status = DISCONNECTED
+            self.set_status(f"Error sending to device {err}", ERRCOL)
 
     def update_ntrip_status(self, status: bool, msgt: tuple = None):
         """
@@ -956,20 +1005,19 @@ class App(Frame):
         elif msgprot == MQTT_PROTOCOL:
             pass
 
-        # update console if visible
-        if widget_state["Console"][VISIBLE]:
-            if msgprot == 0 or msgprot & protfilter:
-                self.frm_console.update_console(raw_data, parsed_data, marker)
+        # update chart data if chart is visible
+        if widget_state[WDGCHART][VISIBLE]:
+            getattr(self, widget_state[WDGCHART][FRAME]).update_data(parsed_data)
+
+        # update consoledata if console is visible and protocol not filtered
+        if widget_state[WDGCONSOLE][VISIBLE] and (msgprot == 0 or msgprot & protfilter):
+            self._consoledata.append((raw_data, parsed_data, marker))
 
         # periodically update widgets if visible
         if datetime.now() > self._last_gui_update + timedelta(
             seconds=GUI_UPDATE_INTERVAL
         ):
-            self.frm_banner.update_frame()
-            for _, widget in widget_state.items():
-                frm = getattr(self, widget["frm"])
-                if hasattr(frm, "update_frame") and widget[VISIBLE]:
-                    frm.update_frame()
+            self._refresh_widgets()
             self._last_gui_update = datetime.now()
 
         # update GPX track file if enabled
@@ -980,6 +1028,20 @@ class App(Frame):
         if settings["datalog_b"]:
             self.file_handler.write_logfile(raw_data, parsed_data)
 
+    def _refresh_widgets(self):
+        """
+        Refresh visible widgets.
+        """
+
+        if widget_state[WDGCONSOLE][VISIBLE]:
+            self.frm_console.update_console(self._consoledata)
+            self._consoledata = []
+        self.frm_banner.update_frame()
+        for _, widget in widget_state.items():
+            frm = getattr(self, widget[FRAME])
+            if hasattr(frm, "update_frame") and widget[VISIBLE]:
+                frm.update_frame()
+
     def _check_update(self):
         """
         Check for updated version.
@@ -987,17 +1049,22 @@ class App(Frame):
 
         latest = check_latest(TITLE)
         if latest not in (VERSION, "N/A"):
-            self.set_status(f"{VERCHECK} {latest}", BADCOL)
+            self.set_status(f"{VERCHECK} {latest}", ERRCOL)
 
-    def poll_version(self):
+    def poll_version(self, protocol: str = "UBX"):
         """
-        Poll MON-VER for device hardware & firmware version.
+        Poll hardware information message for device hardware & firmware version.
+
+        :param str protocol: protocol (UBX)
         """
 
-        msg = UBXMessage("MON", "MON-VER", POLL)
+        if protocol == "NMEA":
+            msg = NMEAMessage("P", "QTMVERNO", POLL)
+        else:
+            msg = UBXMessage("MON", "MON-VER", POLL)
         self.gnss_outqueue.put(msg.serialize())
         self.set_status(
-            "MON-VER POLL message sent",
+            f"{msg.identity} POLL message sent",
         )
 
     @property
