@@ -12,8 +12,7 @@ Created on 13 Sep 2020
 :license: BSD 3-Clause
 """
 
-from platform import system
-from tkinter import SUNKEN, Button, E, Frame, Label, N, S, StringVar, W, font
+from tkinter import SUNKEN, Button, E, Frame, Label, N, S, StringVar, W
 
 from PIL import Image, ImageTk
 from pynmeagps.nmeahelpers import latlon2dmm, latlon2dms, llh2ecef
@@ -30,6 +29,7 @@ from pygpsclient.globals import (
     DMM,
     DMS,
     ECEF,
+    ERRCOL,
     FGCOL,
     ICON_BLANK,
     ICON_CONN,
@@ -47,10 +47,11 @@ from pygpsclient.globals import (
     UIK,
     UMK,
 )
-from pygpsclient.helpers import m2ft, ms2kmph, ms2knots, ms2mph
+from pygpsclient.helpers import m2ft, ms2kmph, ms2knots, ms2mph, scale_font
 
 DGPSYES = "YES"
 DGPSNO = "N/A"
+M2MILES = 5280
 
 
 class BannerFrame(Frame):
@@ -75,6 +76,7 @@ class BannerFrame(Frame):
         self._lat = StringVar()
         self._lon = StringVar()
         self._alt = StringVar()
+        self._hae = StringVar()
         self._speed = StringVar()
         self._alt_u = StringVar()
         self._speed_u = StringVar()
@@ -137,7 +139,10 @@ class BannerFrame(Frame):
             self._frm_basic, text="lon:", bg=self._bgcol, fg=self._fgcol, anchor=N
         )
         self._lbl_lalt = Label(
-            self._frm_basic, text="alt:", bg=self._bgcol, fg=self._fgcol, anchor=N
+            self._frm_basic, text="hmsl:", bg=self._bgcol, fg=self._fgcol, anchor=N
+        )
+        self._lbl_lhae = Label(
+            self._frm_basic, text="hae:", bg=self._bgcol, fg=self._fgcol, anchor=N
         )
         self._lbl_lspd = Label(
             self._frm_basic, text="speed:", bg=self._bgcol, fg=self._fgcol, anchor=N
@@ -198,6 +203,9 @@ class BannerFrame(Frame):
         self._lbl_alt = Label(
             self._frm_basic, textvariable=self._alt, bg=self._bgcol, fg="orange"
         )
+        self._lbl_hae = Label(
+            self._frm_basic, textvariable=self._hae, bg=self._bgcol, fg="orange"
+        )
         self._lbl_spd = Label(
             self._frm_basic, textvariable=self._speed, bg=self._bgcol, fg="deepskyblue"
         )
@@ -217,13 +225,13 @@ class BannerFrame(Frame):
             self._frm_advanced,
             textvariable=self._dop,
             bg=self._bgcol,
-            fg="mediumpurple2",
+            fg="mediumpurple1",
         )
         self._lbl_diffcorr = Label(
             self._frm_advanced,
             textvariable=self._diffcorr,
             bg=self._bgcol,
-            fg="mediumpurple2",
+            fg="hotpink",
         )
 
         self.option_add("*Font", self.__app.font_sm)
@@ -245,7 +253,7 @@ class BannerFrame(Frame):
             self._frm_advanced,
             textvariable=self._hvdop,
             bg=self._bgcol,
-            fg="mediumpurple2",
+            fg="mediumpurple1",
         )
         self._lbl_hvacc = Label(
             self._frm_advanced,
@@ -264,7 +272,7 @@ class BannerFrame(Frame):
             self._frm_advanced,
             textvariable=self._diffstat,
             bg=self._bgcol,
-            fg="mediumpurple2",
+            fg="hotpink",
         )
 
     def _do_layout(self):
@@ -283,14 +291,16 @@ class BannerFrame(Frame):
         self._lbl_lon.grid(column=6, row=0, pady=3, sticky=W)
         self._lbl_lalt.grid(column=7, row=0, pady=3, sticky=W)
         self._lbl_alt.grid(column=8, row=0, pady=3, sticky=W)
-        self._lbl_lalt_u.grid(column=9, row=0, pady=0, sticky=W)
-        self._lbl_lspd.grid(column=10, row=0, pady=3, sticky=W)
-        self._lbl_spd.grid(column=11, row=0, pady=3, sticky=W)
-        self._lbl_lspd_u.grid(column=12, row=0, pady=0, sticky=W)
-        self._lbl_ltrk.grid(column=13, row=0, pady=3, sticky=W)
-        self._lbl_trk.grid(column=14, row=0, pady=3, sticky=W)
-        self._lbl_lfix.grid(column=15, row=0, pady=3, sticky=W)
-        self._lbl_fix.grid(column=16, row=0, pady=3, sticky=W)
+        self._lbl_lhae.grid(column=9, row=0, pady=3, sticky=W)
+        self._lbl_hae.grid(column=10, row=0, pady=3, sticky=W)
+        self._lbl_lalt_u.grid(column=11, row=0, pady=0, sticky=W)
+        self._lbl_lspd.grid(column=12, row=0, pady=3, sticky=W)
+        self._lbl_spd.grid(column=13, row=0, pady=3, sticky=W)
+        self._lbl_lspd_u.grid(column=14, row=0, pady=0, sticky=W)
+        self._lbl_ltrk.grid(column=15, row=0, pady=3, sticky=W)
+        self._lbl_trk.grid(column=16, row=0, pady=3, sticky=W)
+        self._lbl_lfix.grid(column=17, row=0, pady=3, sticky=W)
+        self._lbl_fix.grid(column=18, row=0, pady=3, sticky=W)
 
         self._lbl_lsiv.grid(column=0, row=0, pady=3, sticky=W)
         self._lbl_siv.grid(column=1, row=0, pady=3, sticky=W)
@@ -397,7 +407,7 @@ class BannerFrame(Frame):
         self._update_fix()
         self._update_siv()
         self._update_dop(units)
-        self._update_dgps()
+        self._update_dgps(units)
 
     def _update_time(self):
         """
@@ -406,9 +416,9 @@ class BannerFrame(Frame):
 
         tim = self.__app.gnss_status.utc
         if tim in (None, ""):
-            self._time.set("N/A")
+            self._time.set(f"{'N/A': <15}")
         else:
-            self._time.set(tim)
+            self._time.set(f"{tim:%H:%M:%S.%f}")
 
     def _update_pos(self, pos_format, units):
         """
@@ -421,9 +431,11 @@ class BannerFrame(Frame):
         lat = self.__app.gnss_status.lat
         lon = self.__app.gnss_status.lon
         alt = self.__app.gnss_status.alt  # hMSL
+        hae = self.__app.gnss_status.sep + alt
         self._lbl_llat.config(text="lat:")
         self._lbl_llon.config(text="lon:")
-        self._lbl_lalt.config(text="alt:")
+        self._lbl_lalt.config(text="hmsl:")
+        self._lbl_lhae.config(text="hae:")
         alt_u = "ft" if units in (UI, UIK) else "m"
 
         try:
@@ -443,17 +455,20 @@ class BannerFrame(Frame):
                 elif pos_format == DMM:
                     lat, lon = latlon2dmm(lat, lon)
                 else:
-                    deg_f = ".8f"
+                    deg_f = ".9f"
                 if units in (UI, UIK):
                     alt = m2ft(alt)
+                    hae = m2ft(hae)
                 self._lat.set(f"{lat:{deg_f}}")
                 self._lon.set(f"{lon:{deg_f}}")
             self._alt.set(f"{alt:.4f}")
+            self._hae.set(f"{hae:.4f}")
             self._alt_u.set(f"{alt_u:<2}")
         except (TypeError, ValueError):
             self._lat.set("N/A            ")
             self._lon.set("N/A            ")
             self._alt.set("N/A  ")
+            self._hae.set("N/A  ")
             self._alt_u.set("  ")
 
     def _update_track(self, units):
@@ -497,7 +512,7 @@ class BannerFrame(Frame):
         elif fix in ("2D", "DR"):
             self._lbl_fix.config(fg="orange")
         else:
-            self._lbl_fix.config(fg="red")
+            self._lbl_fix.config(fg=ERRCOL)
         self._fix.set(fix)
 
     def _update_siv(self):
@@ -532,12 +547,34 @@ class BannerFrame(Frame):
                 + f"vacc {self.__app.gnss_status.vacc:.3f}"
             )
 
-    def _update_dgps(self):
+    def _update_dgps(self, units):
         """
         Update DGPS status.
+
+        :param str units: distance units as string (UMM, UMK, UI, UIK)
         """
 
         self._diffcorr.set(DGPSYES if self.__app.gnss_status.diff_corr else DGPSNO)
+        baseline = self.__app.gnss_status.rel_pos_length
+        bl = "N/A"
+        bl_u = ""
+        if isinstance(baseline, (int, float)):
+            if baseline > 0.0:
+                if units in (UI, UIK):
+                    bl = m2ft(baseline / 100)  # cm to ft
+                    if bl > M2MILES:
+                        bl_u = "miles"
+                        bl = bl / M2MILES  # cm to miles
+                    else:
+                        bl_u = "ft"
+                else:
+                    if baseline > 100000:
+                        bl_u = "km"
+                        bl = baseline / 100000  # cm to km
+                    else:
+                        bl_u = "m"
+                        bl = baseline / 100  # cm to m
+
         if self.__app.gnss_status.diff_corr:
             age = self.__app.gnss_status.diff_age
             station = self.__app.gnss_status.diff_station
@@ -547,7 +584,12 @@ class BannerFrame(Frame):
                 age = f"{age} s"
             if station in [None, "", 0]:
                 station = "N/A"
-            self._diffstat.set(f"age {age}\nstation {station}")
+            if bl == "N/A":
+                self._diffstat.set(f"age {age}\nstation {station} baseline {bl}")
+            else:
+                self._diffstat.set(
+                    f"age {age}\nstation {station} baseline {bl:.2f} {bl_u}"
+                )
         else:
             self._diffstat.set("")
 
@@ -557,23 +599,14 @@ class BannerFrame(Frame):
         """
 
         w = self.width
-        # Cater for slightly different font behaviour on Linux
-        if system() in ("Windows", "Darwin"):
-            val = 55
-            lbl = 75
-            sup = 85
-        else:
-            val = 75  # 70
-            lbl = 90  # 85
-            sup = 100  # 95
-
-        sz = min(int(w / val), 18)
+        txt = 100
         for ctl in (
             self._lbl_status_preset,
             self._lbl_time,
             self._lbl_lat,
             self._lbl_lon,
             self._lbl_alt,
+            self._lbl_hae,
             self._lbl_spd,
             self._lbl_trk,
             self._lbl_pdop,
@@ -582,14 +615,15 @@ class BannerFrame(Frame):
             self._lbl_siv,
             self._lbl_diffcorr,
         ):
-            ctl.config(font=font.Font(size=sz))
+            fnt, _ = scale_font(w, 16, txt)
+            ctl.config(font=fnt)
 
-        sz = min(int(w / lbl), 14)
         for ctl in (
             self._lbl_ltime,
             self._lbl_llat,
             self._lbl_llon,
             self._lbl_lalt,
+            self._lbl_lhae,
             self._lbl_lspd,
             self._lbl_ltrk,
             self._lbl_lpdop,
@@ -599,9 +633,9 @@ class BannerFrame(Frame):
             self._lbl_lacc,
             self._lbl_ldgps,
         ):
-            ctl.config(font=font.Font(size=sz))
+            fnt, _ = scale_font(w, 12, txt)
+            ctl.config(font=fnt)
 
-        sz = min(int(w / sup), 12)
         for ctl in (
             self._lbl_lalt_u,
             self._lbl_lspd_u,
@@ -609,7 +643,8 @@ class BannerFrame(Frame):
             self._lbl_hvacc,
             self._lbl_diffstat,
         ):
-            ctl.config(font=font.Font(size=sz))
+            fnt, _ = scale_font(w, 10, txt)
+            ctl.config(font=fnt)
 
     def _on_resize(self, event):  # pylint: disable=unused-argument
         """
@@ -630,6 +665,4 @@ class BannerFrame(Frame):
         """
 
         self.update_idletasks()  # Make sure we know about any resizing
-        width = self.winfo_width()
-        height = self.winfo_height()
-        return (width, height)
+        return self.winfo_width(), self.winfo_height()
