@@ -15,6 +15,7 @@ from pygpsclient.about_dialog import AboutDialog
 from pygpsclient.globals import CFG, CLASS, THD
 from pygpsclient.gpx_dialog import GPXViewerDialog
 from pygpsclient.importmap_dialog import ImportMapDialog
+from pygpsclient.nmea_config_dialog import NMEAConfigDialog
 from pygpsclient.ntrip_client_dialog import NTRIPConfigDialog
 from pygpsclient.spartn_dialog import SPARTNConfigDialog
 from pygpsclient.strings import (
@@ -22,6 +23,7 @@ from pygpsclient.strings import (
     DLGTABOUT,
     DLGTGPX,
     DLGTIMPORTMAP,
+    DLGTNMEA,
     DLGTNTRIP,
     DLGTSPARTN,
     DLGTUBX,
@@ -31,6 +33,7 @@ from pygpsclient.ubx_config_dialog import UBXConfigDialog
 dialog_state = {
     DLGTABOUT: {CLASS: AboutDialog, THD: None, DLG: None, CFG: False},
     DLGTUBX: {CLASS: UBXConfigDialog, THD: None, DLG: None, CFG: True},
+    DLGTNMEA: {CLASS: NMEAConfigDialog, THD: None, DLG: None, CFG: True},
     DLGTNTRIP: {CLASS: NTRIPConfigDialog, THD: None, DLG: None, CFG: True},
     DLGTSPARTN: {CLASS: SPARTNConfigDialog, THD: None, DLG: None, CFG: True},
     DLGTGPX: {CLASS: GPXViewerDialog, THD: None, DLG: None, CFG: True},
