@@ -10,20 +10,42 @@ Created on 14 Sep 2020
 :author: semuadmin
 :copyright: 2020 SEMU Consulting
 :license: BSD 3-Clause
-
 """
 
 # pylint: disable=line-too-long
 
+from collections import namedtuple
 from datetime import datetime
 from os import path
 from pathlib import Path
+from tkinter import Canvas
 
 from pyubx2 import GET, POLL, SET, SETPOLL
 
+Point = namedtuple("Point", ["lat", "lon"])
+# Area convention is minlat, minlon, maxlat, maxlon
+Area = namedtuple("Area", ["lat1", "lon1", "lat2", "lon2"])
+PointXY = namedtuple("Point", ["x", "y"])
+AreaXY = namedtuple("Area", ["x1", "y1", "x2", "y2"])
+
+
+def create_circle(self, x, y, r, **kwargs):
+    """
+    Helper method to simplify drawing circles on canvas
+
+    :param int x: x coordinate
+    :param int y: y coordinate
+    :param int r: radius
+    """
+
+    return self.create_oval(x - r, y - r, x + r, y + r, **kwargs)
+
+
+Canvas.create_circle = create_circle
+
 HOME = Path.home()
 APPNAME = __name__.split(".", 1)[0]  # i.e. "pygpsclient"
-BADCOL = "red"
+AXISCOL = "white"
 BGCOL = "gray24"  # default widget background color
 BPSRATES = (
     9600,
@@ -38,7 +60,6 @@ BPSRATES = (
     4800,
 )
 CFG = "cfg"
-CHECK_FOR_UPDATES = False
 CLASS = "cls"
 COLORTAGS = "colortags"
 CONFIGFILE = path.join(HOME, f"{APPNAME}.json")
@@ -71,7 +92,7 @@ DMM = "DM.M"
 DMS = "D.M.S"
 ECEF = "ECEF"
 ENABLE_CFG_OTHER = True  # enable CFG=* Other Configuration command panel
-ERRCOL = "pink"  # default invalid data entry field background color
+ERRCOL = "salmon"  # default invalid data entry field background color
 ERROR = "ERR!"
 FGCOL = "white"  # default widget foreground color
 FILEREAD_INTERVAL = 0.02  # delay between successive datalog file reads (seconds)
@@ -99,7 +120,7 @@ GNSS_ERR_EVENT = "<<gnss_error>>"
 GNSS_EVENT = "<<gnss_read>>"
 GNSS_TIMEOUT_EVENT = "<<gnss_timeout>>"
 GNSS_LIST = {
-    0: ("GPS", "royalblue"),
+    0: ("GPS", "cornflowerblue"),
     1: ("SBA", "orange"),
     2: ("GAL", "green4"),
     3: ("BEI", "mediumpurple2"),
@@ -117,6 +138,7 @@ GPX_NS = (
     'http://www.topografix.com/GPX/1/1/gpx.xsd"'
 )
 GPX_TRACK_INTERVAL = 1  # minimum GPS track update interval (seconds)
+GRIDCOL = "grey40"
 GUI_UPDATE_INTERVAL = 0.5  # minimum GUI widget update interval (seconds)
 ICON_APP128 = path.join(DIRNAME, "resources/app-128.png")
 ICON_BLANK = path.join(DIRNAME, "resources/blank-1-24.png")
@@ -125,11 +147,13 @@ ICON_CONN = path.join(DIRNAME, "resources/iconmonstr-media-control-48-24.png")
 ICON_CONTRACT = path.join(DIRNAME, "resources/iconmonstr-triangle-1-16.png")
 ICON_DELETE = path.join(DIRNAME, "resources/iconmonstr-trash-can-filled-24.png")
 ICON_DISCONN = path.join(DIRNAME, "resources/iconmonstr-media-control-50-24.png")
+ICON_END = path.join(DIRNAME, "resources/marker_end.png")
 ICON_EXIT = path.join(DIRNAME, "resources/iconmonstr-door-6-24.png")
 ICON_EXPAND = path.join(DIRNAME, "resources/iconmonstr-arrow-80-16.png")
 ICON_GITHUB = path.join(DIRNAME, "resources/github-256.png")
 ICON_LOAD = path.join(DIRNAME, "resources/iconmonstr-folder-18-24.png")
 ICON_LOGREAD = path.join(DIRNAME, "resources/binary-1-24.png")
+ICON_NMEACONFIG = path.join(DIRNAME, "resources/iconmonstr-gear-2-24-brown.png")
 ICON_NOCLIENT = path.join(DIRNAME, "resources/iconmonstr-noclient-10-24.png")
 ICON_NOTRANSMIT = path.join(DIRNAME, "resources/iconmonstr-notransmit-10-24.png")
 ICON_NTRIPCONFIG = path.join(DIRNAME, "resources/iconmonstr-antenna-4-24.png")
@@ -144,6 +168,7 @@ ICON_SEND = path.join(DIRNAME, "resources/iconmonstr-arrow-12-24.png")
 ICON_SERIAL = path.join(DIRNAME, "resources/usbport-1-24.png")
 ICON_SOCKET = path.join(DIRNAME, "resources/ethernet-1-24.png")
 ICON_SPARTNCONFIG = path.join(DIRNAME, "resources/iconmonstr-antenna-3-24.png")
+ICON_START = path.join(DIRNAME, "resources/marker_start.png")
 ICON_STOP = path.join(DIRNAME, "resources/iconmonstr-stop-1-24.png")
 ICON_TRANSMIT = path.join(DIRNAME, "resources/iconmonstr-transmit-10-24.png")
 ICON_UBXCONFIG = path.join(DIRNAME, "resources/iconmonstr-gear-2-24.png")
@@ -152,7 +177,8 @@ ICON_UNKNOWN = path.join(DIRNAME, "resources/clear-1-24.png")
 ICON_WARNING = path.join(DIRNAME, "resources/iconmonstr-warning-1-24.png")
 IMG_WORLD = path.join(DIRNAME, "resources/world.png")
 ICON_SPONSOR = path.join(DIRNAME, "resources/bmc-full-logo-no-background.png")
-IMG_WORLD_CALIB = [90, -180, -90, 180]
+IMG_WORLD_CALIB = Area(-90, -180, 90, 180)
+INFOCOL = "steelblue2"
 KM2M = 1000
 KM2MIL = 0.621371
 KM2NMIL = 0.5399568
@@ -161,20 +187,14 @@ KNOWNGPS = (
     "FT230",
     "FT232",
     "garmin",
-    "Garmin",
     "gnss",
-    "GNSS",
     "gps",
-    "GPS",
     "magellan",
-    "Magellan",
+    "navstar",
+    "septentrio",
     "sirf",
-    "Sirf",
-    "SiRF",
     "trimble",
-    "Trimble",
     "u-blox",
-    "U-Blox",
     "ublox",
     "USB Serial",
     "USB to UART",
@@ -188,6 +208,10 @@ KPH2MPS = 0.2777776918389111005
 LICENSE_URL = "https://github.com/semuconsulting/PyGPSClient/blob/master/LICENSE"
 M2FT = 3.28084
 MAPAPI_URL = "https://developer.mapquest.com/user/login/sign-up"
+CUSTOM = "custom"
+MAP = "map"
+SAT = "sat"
+WORLD = "world"
 MAX_SNR = 60  # upper limit of graphview snr axis
 MAXLOGLINES = 10000  # maximum number of 'lines' per datalog file
 MQAPIKEY = "mqapikey"
@@ -201,6 +225,7 @@ MSGMODES = {
 NOPORTS = 3
 NTRIP_EVENT = "<<ntrip_read>>"
 OKCOL = "green"
+PNTCOL = "orange"
 POPUP_TRANSIENT = True  # whether pop-up config dialogs are always on top
 PORTIDS = ("0 I2C", "1 UART1", "2 UART2", "3 USB", "4 SPI")
 PUBLICIP_URL = "https://ipinfo.io/json"
@@ -209,6 +234,7 @@ QUITONERRORDEFAULT = 1
 RCVR_CONNECTION = "USB,UART1"  # default GNSS receiver connection port(s)
 ROMVER_NEW = "23.01"  # min device ROM version using configuration database
 READONLY = "readonly"
+RPTDELAY = 500  # repeat delay for spinboxes
 RXMMSG = "RXM-SPARTN-KEY"
 SAT_EXPIRY = 10  # how long passed satellites are kept in the sky and graph view
 SAVED_CONFIG = "saved_config"
@@ -225,6 +251,9 @@ SPARTN_EOF_EVENT = "<<spartn_eof>>"
 SPARTN_ERR_EVENT = "<<spartn_error>>"
 SPARTN_EVENT = "<<spartn_read>>"
 SPARTN_KEYLEN = 16
+SPARTN_DEFAULT_KEY = "abcd1234abcd1234abcd1234abcd1234"
+SPARTN_BASEDATE_CURRENT = -1
+SPARTN_BASEDATE_DATASTREAM = 0
 SPARTN_OUTPORT = 8883
 SPARTN_PPREGIONS = ("eu", "us", "jp", "kr", "au")
 SPARTN_PPSERVER_URL = "pp.services.u-blox.com"
@@ -232,7 +261,9 @@ SPARTN_PROTOCOL = 8
 SPARTN_SOURCE_IP = 0
 SPARTN_SOURCE_LB = 1
 SPONSOR_URL = "https://buymeacoffee.com/semuconsulting"
+SQRT2 = 0.7071067811865476  # square root of 2
 THD = "thd"
+TIME0 = datetime(1970, 1, 1)  # basedate for time()
 TIMEOUTS = (
     "0.1",
     "0.2",
@@ -254,14 +285,15 @@ UIK = "Imperial knots"
 UMK = "Metric kmph"
 UMM = "Metric m/s"
 UTF8 = "utf-8"
-WIDGETU1 = (250, 250)  # small widget size
-WIDGETU2 = (350, 250)  # medium widget size
-WIDGETU3 = (950, 350)  # Console size
-WIDGETU4 = (600, 600)  # GPX Track viewer size
+WIDGETU1 = (200, 200)  # small widget size
+WIDGETU2 = (300, 200)  # medium widget size
+WIDGETU3 = (800, 200)  # Console size
+WIDGETU4 = (500, 500)  # GPX Track viewer size
+WIDGETU6 = (400, 200)  # Chart size
 XML_HDR = '<?xml version="1.0" encoding="UTF-8" standalone="yes"?>'
 
-# UBX config widget signifiers - used to
-# identify which widget should receive the
+# UBX & NMEA config widget signifiers - used
+# to identify which widget should receive the
 # data from a given POLL or ACK message:
 UBX_MONVER = 0
 UBX_MONHW = 1
@@ -278,6 +310,9 @@ SPECTRUMVIEW = 11
 SYSMONVIEW = 12
 ROVERVIEW = 13
 UBX_MONRF = 14
+NMEA_MONHW = 15
+NMEA_PRESET = 16
+NMEA_CFGOTHER = 17
 
 # map of fix values to descriptions
 # the keys in this map are a concatenation of NMEA/UBX
