@@ -15,23 +15,37 @@ Created on 17 Apr 2021
 
 import os
 from datetime import datetime, timedelta
-from math import atan, cos, pi, sin, sqrt, trunc
-from platform import system
+from math import asin, atan, atan2, cos, degrees, pi, radians, sin, sqrt, trunc
 from socket import AF_INET, SOCK_DGRAM, socket
 from time import strftime
-from tkinter import Button, Entry, Label, Toplevel, W, font
+from tkinter import Entry
+from tkinter.font import Font
 
-from pynmeagps import haversine
-from pyubx2 import SET, SET_LAYER_RAM, TXN_NONE, UBX_MSGIDS, UBXMessage, attsiz, atttyp
+from pynmeagps import WGS84_SMAJ_AXIS, haversine
+from pyubx2 import (
+    SET,
+    SET_LAYER_RAM,
+    TXN_NONE,
+    UBX_CLASSES,
+    UBX_MSGIDS,
+    UBXMessage,
+    attsiz,
+    atttyp,
+)
 from requests import get
 
 from pygpsclient.globals import (
+    ERRCOL,
     FIXLOOKUP,
     GPSEPOCH0,
     MAX_SNR,
     PUBLICIP_URL,
     RCVR_CONNECTION,
     ROMVER_NEW,
+    TIME0,
+    Area,
+    AreaXY,
+    Point,
 )
 from pygpsclient.strings import NA
 
@@ -54,86 +68,7 @@ NMEA = {"0": "No GGA", "1": "GGA"}
 AUTHS = {"N": "None", "B": "Basic", "D": "Digest"}
 CARRIERS = {"0": "No", "1": "L1", "2": "L1,L2"}
 SOLUTIONS = {"0": "Single", "1": "Network"}
-
-
-class ConfirmBox(Toplevel):
-    """
-    Confirm action dialog class.
-    Provides better consistency across different OS platforms
-    than using messagebox.askyesno()
-
-    Returns True if OK, False if Cancel
-    """
-
-    def __init__(self, parent, title, prompt):
-        """
-        Constructor
-
-        :param parent: parent dialog
-        :param string title: title
-        :param string prompt: prompt to be displayed
-        """
-
-        self.__master = parent
-        Toplevel.__init__(self, parent)
-        self.title(title)  # pylint: disable=E1102
-        self.resizable(False, False)
-        Label(self, text=prompt, anchor=W).grid(
-            row=0, column=0, columnspan=2, padx=3, pady=5
-        )
-        Button(self, command=self._on_ok, text="OK", width=8).grid(
-            row=1, column=0, padx=3, pady=3
-        )
-        Button(self, command=self._on_cancel, text="Cancel", width=8).grid(
-            row=1, column=1, padx=3, pady=3
-        )
-        self.lift()  # Put on top of
-        self.grab_set()  # Make modal
-        self._rc = False
-
-        self._centre()
-
-    def _on_ok(self, event=None):  # pylint: disable=unused-argument
-        """
-        OK button handler
-        """
-
-        self._rc = True
-        self.destroy()
-
-    def _on_cancel(self, event=None):  # pylint: disable=unused-argument
-        """
-        Cancel button handler
-        """
-
-        self._rc = False
-        self.destroy()
-
-    def _centre(self):
-        """
-        Centre dialog in parent
-        """
-
-        # self.update_idletasks()
-        dw = self.winfo_width()
-        dh = self.winfo_height()
-        mx = self.__master.winfo_x()
-        my = self.__master.winfo_y()
-        mw = self.__master.winfo_width()
-        mh = self.__master.winfo_height()
-        self.geometry(f"+{int(mx + (mw/2 - dw/2))}+{int(my + (mh/2 - dh/2))}")
-
-    def show(self):
-        """
-        Show dialog
-
-        :return: True (OK) or False (Cancel)
-        :rtype: bool
-        """
-
-        self.wm_deiconify()
-        self.wait_window()
-        return self._rc
+POINTLIMIT = 500  # max number of shape points supported by MapQuest API
 
 
 def cel2cart(elevation: float, azimuth: float) -> tuple:
@@ -604,7 +539,7 @@ def valid_entry(entry: Entry, valmode: int, low=MINFLOAT, high=MAXFLOAT) -> bool
         entry.configure(highlightthickness=0)
     else:
         entry.configure(
-            highlightthickness=2, highlightbackground="red", highlightcolor="red"
+            highlightthickness=2, highlightbackground=ERRCOL, highlightcolor=ERRCOL
         )
     return valid
 
@@ -633,6 +568,10 @@ def stringvar2val(val: str, att: str) -> object:
         val = bytes(val, "utf-8")
     elif atttyp(att) == "R":  # float
         val = float(val)
+    elif atttyp(att) in ("IN", "HX"):  # NMEA int
+        val = int(val)
+    elif atttyp(att) in ("DE", "LA", "LN"):  # NMEA float
+        val = float(val)
 
     return val
 
@@ -781,24 +720,50 @@ def secs2unit(secs: int) -> tuple:
     return val, SECSUNITS[i]
 
 
-def sizefont(height: int, lines: int, minfont: int) -> tuple:
+def fontwidth(fnt: Font, txt: str = "W") -> int:
     """
-    Set font size according to number of text lines on widget
-    of given height.
+    Get font width.
 
-    :param int maxlines: max no of lines of text
-    :param int minfont: min font size
-    :returns: tuple of (font, fontheight)
+    :param Font fnt:font
+    :param txt: reference text ("W")
+    :return: font width in pixels
+    :rtype: int
+    """
+
+    return Font.measure(fnt, txt)
+
+
+def fontheight(fnt: Font) -> int:
+    """
+    Get font height.
+
+    :param Font fnt: font
+    :return: font height in pixels
+    :rtype: int
+    """
+
+    return Font.metrics(fnt, "linespace")
+
+
+def scale_font(
+    width: int, basesize: int, txtwidth: int, maxsize: int = 0, fnt: Font = None
+) -> tuple:
+    """
+    Scale font size to widget width.
+
+    :param int width: widget width
+    :param int bassiz: base font size
+    :param int txtwidth: reference text width
+    :param int maxsiz: max font size
+    :param Font fnt: default font
+    :return: tuple of scaled font, font height
     :rtype: tuple
     """
 
-    fh = 0
-    fs = minfont
-    while fh * lines < height:
-        fnt = font.Font(size=fs)
-        fh = fnt.metrics("linespace")
-        fs += 1
-    return fnt, fh
+    fnt = Font(size=12) if fnt is None else fnt
+    fs = basesize * width / fontwidth(fnt, "W" * txtwidth)
+    fnt = Font(size=int(min(fs, maxsize))) if maxsize else Font(size=int(fs))
+    return fnt, fontheight(fnt)
 
 
 def setubxrate(app: object, mid: str, rate: int = 1, prot: str = "UBX") -> UBXMessage:
@@ -918,22 +883,6 @@ def config_nmea(state: int, port_type: str = "USB") -> UBXMessage:
     return UBXMessage.config_set(layers, transaction, cfg_data)
 
 
-def adjust_dimensions(dim: int) -> int:
-    """
-    Adjust display dimensions for different operating systems.
-
-    :param int dim: dimension
-    :return: adjusted dimension
-    :rtype: int
-    """
-
-    if system() == "Windows":
-        return int(dim * 0.95)
-    if system() == "Darwin":
-        return int(dim * 0.93)
-    return int(dim)
-
-
 def ned2vector(n: float, e: float, d: float) -> tuple:
     """
     Convert N,E,D relative position to 2D heading and distance.
@@ -1005,3 +954,331 @@ def lanip() -> str:
             return sck.getsockname()[0]
         except Exception:  # pylint: disable=broad-exception-caught
             return "N/A"
+
+
+def get_point_at_vector(
+    start: Point,
+    dist: float,
+    bearing: float,
+    radius: float = WGS84_SMAJ_AXIS,
+) -> Point:
+    """
+    Get new point at vector from start position.
+
+    :param Point start: starting position
+    :param float dist: vector distance
+    :param float bearing: vector bearing (true)
+    :param float radius: optional radius of sphere, defaults to mean radius of earth
+    :return: new position as lat/lon
+    :rtype: Point
+    """
+
+    phi1 = radians(start.lat)
+    lambda1 = radians(start.lon)
+    br = radians(bearing)
+    phi2 = asin(
+        sin(phi1) * cos(dist / radius) + cos(phi1) * sin(dist / radius) * cos(br)
+    )
+    lambda2 = lambda1 + atan2(
+        sin(br) * sin(dist / radius) * cos(phi1),
+        cos(dist / radius) - sin(phi1) * sin(phi2),
+    )
+    return Point(degrees(phi2), degrees(lambda2))
+
+
+def in_bounds(bounds: Area, point: Point) -> bool:
+    """
+    Check if given point is within canvas bounding box.
+
+    :param Area bounds: bounding box
+    :param Point point: point
+    :return: true/false
+    :rtype: bool
+    """
+
+    if bounds is None:
+        return False
+
+    return (
+        point.lat >= bounds.lat1
+        and point.lat <= bounds.lat2
+        and point.lon >= bounds.lon1
+        and point.lon <= bounds.lon2
+    )
+
+
+def reorder_range(valuerange: tuple, default) -> tuple:
+    """
+    Reorder range so default is first value (e.g. for
+    spinbox objects).
+
+    :param tuple valuerange: range
+    :param object default: starting value
+    :return: range starting with default
+    :rtype: tuple
+    """
+
+    drange = valuerange
+    lr = len(valuerange)
+    for i, val in enumerate(valuerange):
+        if val == default:
+            drange = valuerange[i:lr] + valuerange[0:i]
+            break
+    return drange
+
+
+def ll2xy(width: int, height: int, bounds: Area, position: Point) -> tuple:
+    """
+    Convert lat/lon to canvas x/y.
+
+    :param int width: canvas width
+    :param int height: canvas height
+    :param Area bounds: lat/lon bounds of canvas
+    :param Point coordinate: lat/lon
+    :return: x,y canvas coordinates
+    :rtype: tuple
+    """
+
+    lw = bounds.lon2 - bounds.lon1
+    lh = bounds.lat2 - bounds.lat1
+    x = (position.lon - bounds.lon1) / (lw / width)
+    y = height - (position.lat - bounds.lat1) / (lh / height)
+    return x, y
+
+
+def xy2ll(width: int, height: int, bounds: Area, xy: tuple) -> Point:
+    """
+    Convert canvas x/y to lat/lon.
+
+    :param int width: canvas width
+    :param int height: canvas height
+    :param Area bounds: lat/lon bounds of canvas
+    :param tuple xy: canvas x/y coordinate
+    :return: lat/lon
+    :rtype: Point
+    """
+
+    lw = bounds.lon2 - bounds.lon1
+    lh = bounds.lat2 - bounds.lat1
+    x, y = xy
+    lon = bounds.lon1 + x / (width / lw)
+    lat = bounds.lat1 + (height - y) / (height / lh)
+    return Point(lat, lon)
+
+
+def data2xy(
+    width: int,
+    height: int,
+    bounds: AreaXY,
+    xdata: float,
+    ydata: float,
+    xoffset: float = 0,
+    yoffset: float = 0,
+) -> tuple:
+    """
+    Convert datapoint x,y to canvas x,y. Y is vertical axis.
+
+    :param int width: canvas width
+    :param int height: canvas height
+    :param AreaXY bounds: x,y bounds of data
+    :param float xdata: datapoint x
+    :param float ydata: datapoint y
+    :param float xoffset: canvas x offset
+    :param float yoffset: canvas y offset
+    :return: x,y canvas coordinates
+    :rtype: tuple
+    """
+    # pylint: disable=too-many-arguments, too-many-positional-arguments
+
+    try:
+        lw = bounds.x2 - bounds.x1
+        lh = bounds.y2 - bounds.y1
+        x = (xdata - bounds.x1) / (lw / width)
+        y = height - (ydata - bounds.y1) / (lh / height)
+    except ZeroDivisionError:
+        return 0, 0
+    return x + xoffset, y + yoffset
+
+
+def xy2data(
+    width: int,
+    height: int,
+    bounds: AreaXY,
+    x: int,
+    y: int,
+    xoffset: int = 0,
+    yoffset: int = 0,
+) -> tuple:
+    """
+    Convert canvas x,y to datapoint x,y. Y is vertical axis.
+
+    :param int width: canvas width
+    :param int height: canvas height
+    :param AreaXY bounds: x,y bounds of data
+    :param int x: canvas x coordinate
+    :param int y: canvas y coordinate
+    :param float xoffset: canvas x offset
+    :param float yoffset: canvas y offset
+    :return: xdata, ydata
+    :rtype: tuple
+    """
+    # pylint: disable=too-many-arguments, too-many-positional-arguments
+
+    try:
+        lw = bounds.x2 - bounds.x1
+        lh = bounds.y2 - bounds.y1
+        datax = bounds.x1 + (x - xoffset) / (width / lw)
+        datay = bounds.y1 + (height - (y - yoffset)) / (height / lh)
+    except ZeroDivisionError:
+        return 0, 0
+    return datax, datay
+
+
+def points2area(points: tuple) -> Area:
+    """
+    Convert 4 points to Area.
+
+    Points in order (minlat, minlon, maxlat, maxlon)
+
+    :param tuple points: tuple of points
+    :raises TypeError: if less than 4 points provided
+    :return: area
+    :rtype Area
+    """
+
+    if len(points) != 4:
+        raise ValueError("Exactly 4 points required")
+
+    minlat = min(points[0], points[2])
+    maxlat = max(points[0], points[2])
+    minlon = min(points[1], points[3])
+    maxlon = max(points[1], points[3])
+
+    return Area(minlat, minlon, maxlat, maxlon)
+
+
+def limittrack(track: list, limit: int = POINTLIMIT) -> list:
+    """
+    Limit number of points in track.
+
+    :param list track: list of Points
+    :param int limit: max points
+    :return: limited list of Points
+    """
+
+    points = []
+    stp = 1
+    rng = len(track)
+    while rng / stp > limit:
+        stp += 1
+    for i, p in enumerate(track):
+        if i % stp == 0:
+            points.append(p)
+    return points
+
+
+def get_grid(
+    num: int = 10, start: int = 0, stop: int = 1, endpoint: bool = True
+) -> tuple:
+    """
+    Generate linear grid steps for graphing widgets.
+
+    :param int num: number of increments (10)
+    :param int start: start point (0)
+    :param int stop: end point (1)
+    :param bool endpoint: include endpoint (True)
+    :return: linear grid increments
+    :rtype: tuple
+    """
+
+    def linspace(start, stop, num, endpoint):
+        """Generator for linear grid"""
+        num = int(num)
+        start = start * 1.0
+        stop = stop * 1.0
+
+        if num == 1:
+            yield round(stop, 4)
+            return
+        if endpoint:
+            step = (stop - start) / (num - 1)
+        else:
+            step = (stop - start) / num
+
+        for i in range(num):
+            yield round(start + step * i, 4)
+
+    return tuple(linspace(start, stop, num, endpoint))
+
+
+def time2str(tim: float, sformat: str = "%H:%M:%S") -> str:
+    """
+    Convert time float to formatted string.
+
+    :param float tim: time as float (seconds since 1970-01-01-00:00:00)
+    :param str sformat: string format ("%H:%M:%S")
+    :return: formated time string
+    :rtype: str
+    """
+
+    dt = TIME0 + timedelta(seconds=tim)
+    return dt.strftime(sformat)
+
+
+def ubx2preset(msgs: tuple, desc: str = "") -> str:
+    """
+    Convert one or more UBXMessages to format suitable for adding to user-defined
+    preset list `ubxpresets_l` in PyGPSClient *.json configuration files.
+
+    The format is:
+    "<description>, <talker>, <msgID>, <payload as hexadecimal string>, <msgmode>"
+
+    e.g. "Set NMEA High Precision Mode, CFG, CFG-VALSET, 000100000600931001, 1"
+
+    :param tuple msgs: UBXMessage or tuple of UBXmessages
+    :param str desc: preset description
+    :return: preset string
+    :rtype: str
+    """
+
+    if not isinstance(msgs, tuple):
+        msgs = (msgs,)
+    preset = (
+        f"{msgs[0].identity} {['GET','SET','POLL'][msgs[0].msgmode]}"
+        if desc == ""
+        else desc
+    )
+    for msg in msgs:
+        preset += (
+            f", {UBX_CLASSES[msg.msg_cls]}, {UBX_MSGIDS[msg.msg_cls + msg.msg_id]}, "
+            f"{msg.payload.hex()}, {msg.msgmode}"
+        )
+    return preset
+
+
+def nmea2preset(msgs: tuple, desc: str = "") -> str:
+    """
+    Convert one or more NMEAMessages to format suitable for adding to user-defined
+    preset list `nmeapresets_l` in PyGPSClient *.json configuration files.
+
+    The format is:
+    "<description>; <talker>; <msgID>; <payload as comma separated list>; <msgmode>"
+
+    e.g. "Configure Signals; P; QTMCFGSIGNAL; W,7,3,F,3F,7,1; 1"
+
+    :param tuple msgs: NMEAmessage or tuple of NMEAmessages
+    :param str desc: preset description
+    :return: preset string
+    :rtype: str
+    """
+
+    if not isinstance(msgs, tuple):
+        msgs = (msgs,)
+    preset = (
+        f"{msgs[0].identity} {['GET','SET','POLL'][msgs[0].msgmode]}"
+        if desc == ""
+        else desc
+    )
+    for msg in msgs:
+        preset += f"; {msg.talker}; {msg.msgID}; {','.join(msg.payload)}; {msg.msgmode}"
+    return preset
