@@ -52,12 +52,15 @@ from pygnssutils.helpers import find_mp_distance
 from pygpsclient.globals import (
     CONNECTED_NTRIP,
     DISCONNECTED,
+    ERRCOL,
     GGA_INTERVALS,
     ICON_CONN,
     ICON_DISCONN,
     ICON_EXIT,
+    INFOCOL,
     POPUP_TRANSIENT,
     READONLY,
+    RPTDELAY,
     SAVED_CONFIG,
     UBX_CFGMSG,
     UBX_CFGPRT,
@@ -180,14 +183,12 @@ class NTRIPConfigDialog(Toplevel):
             server_callback=self._on_server,
         )
         self._frm_status = Frame(self._frm_container, borderwidth=2, relief="groove")
-        self._lbl_status = Label(
-            self._frm_status, textvariable=self._status, anchor="w"
-        )
+        self._lbl_status = Label(self._frm_status, textvariable=self._status, anchor=W)
         self._btn_exit = Button(
             self._frm_status,
             image=self._img_exit,
             width=55,
-            fg="red",
+            fg=ERRCOL,
             command=self.on_exit,
             font=self.__app.font_md,
         )
@@ -204,7 +205,7 @@ class NTRIPConfigDialog(Toplevel):
             self._frm_container,
             textvariable=self._ntrip_mpdist,
             width=30,
-            anchor="w",
+            anchor=W,
         )
         self._lbl_sourcetable = Label(self._frm_container, text=LBLNTRIPSTR)
         self._lbx_sourcetable = Listbox(
@@ -226,6 +227,8 @@ class NTRIPConfigDialog(Toplevel):
             values=(NTRIP_VERSIONS),
             width=4,
             wrap=True,
+            repeatdelay=RPTDELAY,
+            repeatinterval=RPTDELAY,
             textvariable=self._ntrip_version,
             state=READONLY,
         )
@@ -235,6 +238,8 @@ class NTRIPConfigDialog(Toplevel):
             values=(RTCM, SPARTN),
             width=8,
             wrap=True,
+            repeatdelay=RPTDELAY,
+            repeatinterval=RPTDELAY,
             textvariable=self._ntrip_datatype,
             state=READONLY,
         )
@@ -261,6 +266,8 @@ class NTRIPConfigDialog(Toplevel):
             values=(GGA_INTERVALS),
             width=5,
             wrap=True,
+            repeatdelay=RPTDELAY,
+            repeatinterval=RPTDELAY,
             textvariable=self._ntrip_gga_interval,
             state=READONLY,
         )
@@ -452,7 +459,7 @@ class NTRIPConfigDialog(Toplevel):
                     else "Disconnected"
                 )
             if msgt is None:
-                self.set_status(msg, "blue")
+                self.set_status(msg, INFOCOL)
             else:
                 msg, col = msgt
                 self.set_status(msg, col)
@@ -505,6 +512,7 @@ class NTRIPConfigDialog(Toplevel):
         :param str color: rgb color of text
         """
 
+        color = INFOCOL if color == "blue" else color
         message = f"{message[:78]}.." if len(message) > 80 else message
         if color != "":
             self._lbl_status.config(fg=color)
@@ -703,7 +711,7 @@ class NTRIPConfigDialog(Toplevel):
             valid = valid & valid_entry(self._ent_sep, VALFLOAT, -MAXALT, MAXALT)
 
         if not valid:
-            self.set_status("ERROR - invalid settings", "red")
+            self.set_status("ERROR - invalid settings", ERRCOL)
 
         return valid
 
