@@ -45,9 +45,12 @@ from pygpsclient.globals import (
     DEFAULT_PASSWORD,
     DEFAULT_USER,
     DISCONNECTED,
+    ERRCOL,
     ICON_CONTRACT,
     ICON_EXPAND,
+    INFOCOL,
     READONLY,
+    RPTDELAY,
     SAVED_CONFIG,
     SOCK_NTRIP,
     SOCKMODES,
@@ -97,6 +100,16 @@ ACCURACIES = (
     30,
     20,
 )
+RTCMTYPES = (
+    "1006",
+    "1077",
+    "1087",
+    "1097",
+    "1127",
+    "1230",
+    "4072_0",
+    "4072_1",
+)
 BASE_DISABLED = "DISABLED"
 BASE_FIXED = "FIXED"
 BASE_SVIN = "SURVEY IN"
@@ -184,6 +197,8 @@ class ServerConfigFrame(Frame):
             width=14,
             state=READONLY,
             wrap=True,
+            repeatdelay=RPTDELAY,
+            repeatinterval=RPTDELAY,
             textvariable=self.sock_mode,
         )
         self._lbl_sockhost = Label(
@@ -267,6 +282,8 @@ class ServerConfigFrame(Frame):
             width=10,
             state=READONLY,
             wrap=True,
+            repeatdelay=RPTDELAY,
+            repeatinterval=RPTDELAY,
             textvariable=self.base_mode,
         )
         self._lbl_acclimit = Label(
@@ -279,6 +296,8 @@ class ServerConfigFrame(Frame):
             width=5,
             state=READONLY,
             wrap=True,
+            repeatdelay=RPTDELAY,
+            repeatinterval=RPTDELAY,
             textvariable=self.acclimit,
         )
         self._lbl_duration = Label(
@@ -296,6 +315,8 @@ class ServerConfigFrame(Frame):
             width=5,
             state=READONLY,
             wrap=True,
+            repeatdelay=RPTDELAY,
+            repeatinterval=RPTDELAY,
             textvariable=self.duration,
         )
         self._lbl_elapsed = Label(
@@ -314,6 +335,8 @@ class ServerConfigFrame(Frame):
             width=6,
             state=READONLY,
             wrap=True,
+            repeatdelay=RPTDELAY,
+            repeatinterval=RPTDELAY,
             textvariable=self.pos_mode,
         )
         self._lbl_fixedlat = Label(
@@ -451,9 +474,9 @@ class ServerConfigFrame(Frame):
             valid = valid & valid_entry(self._ent_fixedlon, VALFLOAT)
             valid = valid & valid_entry(self._ent_fixedalt, VALFLOAT)
             if valid:
-                self.__app.set_status("", "blue")
+                self.__app.set_status("", INFOCOL)
             else:
-                self.__app.set_status("ERROR - invalid entry", "red")
+                self.__app.set_status("ERROR - invalid entry", ERRCOL)
                 self._socket_serve.set(0)
                 return
             # start server
@@ -723,16 +746,8 @@ class ServerConfigFrame(Frame):
         layers = 1  # 1 = RAM, 2 = BBR, 4 = Flash (can be OR'd)
         transaction = 0
         cfg_data = []
-        for rtcm_type in (
-            "1005",
-            "1077",
-            "1087",
-            "1097",
-            "1127",
-            "1230",
-            "4072_0",
-            "4072_1",
-        ):
+        for rtcm_type in RTCMTYPES:
+
             cfg = f"CFG_MSGOUT_RTCM_3X_TYPE{rtcm_type}_{port_type}"
             cfg_data.append([cfg, rate])
 
