@@ -19,6 +19,7 @@ Created on 12 Sep 2020
 
 # pylint: disable=unnecessary-lambda
 
+from platform import system
 from socket import AF_INET6
 from tkinter import (
     ALL,
@@ -28,6 +29,7 @@ from tkinter import (
     HORIZONTAL,
     LEFT,
     NORMAL,
+    NW,
     RIGHT,
     VERTICAL,
     Button,
@@ -53,16 +55,17 @@ from pyubx2 import ubxtypes_core as ubt
 from serial import PARITY_NONE
 
 from pygpsclient.globals import (
-    BADCOL,
     BPSRATES,
     CONNECTED,
     CONNECTED_FILE,
     CONNECTED_SOCKET,
+    CUSTOM,
     DDD,
     DISCONNECTED,
     DMM,
     DMS,
     ECEF,
+    ERRCOL,
     FORMAT_BINARY,
     FORMAT_PARSED,
     FORMATS,
@@ -70,43 +73,55 @@ from pygpsclient.globals import (
     GNSS_ERR_EVENT,
     GNSS_EVENT,
     GNSS_TIMEOUT_EVENT,
+    HOME,
     ICON_CONN,
     ICON_DISCONN,
     ICON_EXIT,
     ICON_LOGREAD,
+    ICON_NMEACONFIG,
     ICON_NTRIPCONFIG,
     ICON_SERIAL,
     ICON_SOCKET,
     ICON_SPARTNCONFIG,
     ICON_UBXCONFIG,
     KNOWNGPS,
+    MAP,
     MSGMODES,
     NOPORTS,
+    OKCOL,
     RCVR_CONNECTION,
     READONLY,
+    RPTDELAY,
+    SAT,
     SOCK_NTRIP,
+    SPARTN_BASEDATE_CURRENT,
+    SPARTN_DEFAULT_KEY,
     SPARTN_PROTOCOL,
     TIMEOUTS,
     UI,
     UIK,
     UMK,
     UMM,
+    WORLD,
 )
-from pygpsclient.helpers import adjust_dimensions
+from pygpsclient.helpers import fontheight, fontwidth
 from pygpsclient.mapquest import MAP_UPDATE_INTERVAL
 from pygpsclient.serialconfig_frame import SerialConfigFrame
 from pygpsclient.serverconfig_frame import ServerConfigFrame
 from pygpsclient.socketconfig_frame import SocketConfigFrame
 from pygpsclient.strings import (
     DLG,
+    DLGTNMEA,
     DLGTNTRIP,
     DLGTSPARTN,
     DLGTUBX,
     LBLDATADISP,
     LBLDATALOG,
     LBLDEGFORMAT,
+    LBLNMEACONFIG,
     LBLNTRIPCONFIG,
     LBLPROTDISP,
+    LBLSHOWTRACK,
     LBLSHOWUNUSED,
     LBLSPARTNCONFIG,
     LBLTRACKRECORD,
@@ -114,9 +129,19 @@ from pygpsclient.strings import (
 )
 
 MAXLINES = ("200", "500", "1000", "2000", "100")
-MAPTYPES = ("world", "map", "sat", "custom")
-MINHEIGHT = 750
-MINWIDTH = 390
+MAPTYPES = (WORLD, MAP, SAT, CUSTOM)
+# initial dimensions adjusted for different widget
+# rendering on different platforms
+if system() == "Linux":  # Wayland
+    MINHEIGHT = 28
+    MINWIDTH = 28
+elif system() == "Darwin":  # MacOS
+
+    MINHEIGHT = 38
+    MINWIDTH = 30
+else:  # Windows and others
+    MINHEIGHT = 35
+    MINWIDTH = 26
 
 
 class SettingsFrame(Frame):
@@ -139,7 +164,8 @@ class SettingsFrame(Frame):
         Frame.__init__(self, self.__master, *args, **kwargs)
 
         self.infilepath = None
-        self.outfilepath = None
+        self.logpath = HOME
+        self.trackpath = HOME
         self._prot_nmea = IntVar()
         self._prot_ubx = IntVar()
         self._prot_rtcm3 = IntVar()
@@ -147,6 +173,7 @@ class SettingsFrame(Frame):
         self._autoscroll = IntVar()
         self._maxlines = IntVar()
         self.maptype = StringVar()
+        self.showtrack = IntVar()
         self.mapzoom = IntVar()
         self._units = StringVar()
         self._degrees_format = StringVar()
@@ -161,13 +188,13 @@ class SettingsFrame(Frame):
             "defaultport_s", RCVR_CONNECTION
         )
         self._validsettings = True
-        self._trackpath = None
         self._img_conn = ImageTk.PhotoImage(Image.open(ICON_CONN))
         self._img_serial = ImageTk.PhotoImage(Image.open(ICON_SERIAL))
         self._img_socket = ImageTk.PhotoImage(Image.open(ICON_SOCKET))
         self._img_disconn = ImageTk.PhotoImage(Image.open(ICON_DISCONN))
         self._img_exit = ImageTk.PhotoImage(Image.open(ICON_EXIT))
         self._img_ubxconfig = ImageTk.PhotoImage(Image.open(ICON_UBXCONFIG))
+        self._img_nmeaconfig = ImageTk.PhotoImage(Image.open(ICON_NMEACONFIG))
         self._img_ntripconfig = ImageTk.PhotoImage(Image.open(ICON_NTRIPCONFIG))
         self._img_spartnconfig = ImageTk.PhotoImage(Image.open(ICON_SPARTNCONFIG))
         self._img_dataread = ImageTk.PhotoImage(Image.open(ICON_LOGREAD))
@@ -185,7 +212,8 @@ class SettingsFrame(Frame):
         function which invokes the on_expand() method here.
         """
 
-        dimw, dimh = [adjust_dimensions(x) for x in (MINWIDTH, MINHEIGHT)]
+        dimw = fontwidth(self.__app.font_md) * MINWIDTH
+        dimh = fontheight(self.__app.font_md) * MINHEIGHT
         self._frm_main = Frame(self)
         self._frm_main.pack(fill=BOTH, expand=1)
         self_frm_scrollx = Frame(self._frm_main)
@@ -203,9 +231,7 @@ class SettingsFrame(Frame):
         y_scrollbar.pack(side=RIGHT, fill=Y)
         self._can_container.configure(xscrollcommand=x_scrollbar.set)
         self._can_container.configure(yscrollcommand=y_scrollbar.set)
-        self._can_container.create_window(
-            (0, 0), window=self._frm_container, anchor="nw"
-        )
+        self._can_container.create_window((0, 0), window=self._frm_container, anchor=NW)
         self._can_container.bind(
             "<Configure>",
             lambda e: self._can_container.config(
@@ -324,6 +350,8 @@ class SettingsFrame(Frame):
             width=10,
             state=READONLY,
             wrap=True,
+            repeatdelay=RPTDELAY,
+            repeatinterval=RPTDELAY,
             textvariable=self._console_format,
         )
         self._chk_tags = Checkbutton(
@@ -338,6 +366,8 @@ class SettingsFrame(Frame):
             width=6,
             state=READONLY,
             wrap=True,
+            repeatdelay=RPTDELAY,
+            repeatinterval=RPTDELAY,
             textvariable=self._degrees_format,
         )
         self._spn_units = Spinbox(
@@ -346,6 +376,8 @@ class SettingsFrame(Frame):
             width=13,
             state=READONLY,
             wrap=True,
+            repeatdelay=RPTDELAY,
+            repeatinterval=RPTDELAY,
             textvariable=self._units,
         )
         self._chk_scroll = Checkbutton(
@@ -356,6 +388,8 @@ class SettingsFrame(Frame):
             values=MAXLINES,
             width=6,
             wrap=True,
+            repeatdelay=RPTDELAY,
+            repeatinterval=RPTDELAY,
             textvariable=self._maxlines,
             state=READONLY,
         )
@@ -365,9 +399,14 @@ class SettingsFrame(Frame):
             values=MAPTYPES,
             width=6,
             wrap=True,
+            repeatdelay=RPTDELAY,
+            repeatinterval=RPTDELAY,
             textvariable=self.maptype,
             state=READONLY,
         )
+        self._chk_showtrack = Checkbutton(
+            self._frm_options, text=LBLSHOWTRACK, variable=self.showtrack
+        )
         self._chk_unusedsat = Checkbutton(
             self._frm_options, text=LBLSHOWUNUSED, variable=self._show_unusedsat
         )
@@ -382,6 +421,8 @@ class SettingsFrame(Frame):
             values=(FORMATS),
             width=20,
             wrap=True,
+            repeatdelay=RPTDELAY,
+            repeatinterval=RPTDELAY,
             textvariable=self._logformat,
             state=READONLY,
         )
@@ -402,6 +443,16 @@ class SettingsFrame(Frame):
             image=self._img_ubxconfig,
             command=lambda: self._on_ubx_config(),
         )
+        self._lbl_nmeaconfig = Label(
+            self._frm_options,
+            text=LBLNMEACONFIG,
+        )
+        self._btn_nmeaconfig = Button(
+            self._frm_options,
+            width=45,
+            image=self._img_nmeaconfig,
+            command=lambda: self._on_nmea_config(),
+        )
         self._lbl_ntripconfig = Label(
             self._frm_options,
             text=LBLNTRIPCONFIG,
@@ -478,6 +529,7 @@ class SettingsFrame(Frame):
         self._chk_scroll.grid(column=0, row=5, padx=2, pady=2, sticky=W)
         self._spn_maxlines.grid(column=1, row=5, columnspan=3, padx=2, pady=2, sticky=W)
         self._lbl_maptype.grid(column=0, row=6, padx=2, pady=2, sticky=W)
+        self._chk_showtrack.grid(column=2, row=6, padx=2, pady=2, sticky=W)
         self.spn_maptype.grid(column=1, row=6, padx=2, pady=2, sticky=W)
         self._chk_unusedsat.grid(
             column=0, row=7, columnspan=2, padx=2, pady=2, sticky=W
@@ -489,10 +541,12 @@ class SettingsFrame(Frame):
         )
         self._btn_ubxconfig.grid(column=0, row=10)
         self._lbl_ubxconfig.grid(column=0, row=11)
-        self._btn_ntripconfig.grid(column=1, row=10)
-        self._lbl_ntripconfig.grid(column=1, row=11)
-        self._btn_spartnconfig.grid(column=2, row=10)
-        self._lbl_spartnconfig.grid(column=2, row=11)
+        self._btn_nmeaconfig.grid(column=1, row=10)
+        self._lbl_nmeaconfig.grid(column=1, row=11)
+        self._btn_ntripconfig.grid(column=2, row=10)
+        self._lbl_ntripconfig.grid(column=2, row=11)
+        self._btn_spartnconfig.grid(column=3, row=10)
+        self._lbl_spartnconfig.grid(column=3, row=11)
         ttk.Separator(self._frm_container).grid(
             column=0, row=9, columnspan=4, padx=2, pady=2, sticky=(W, E)
         )
@@ -505,7 +559,6 @@ class SettingsFrame(Frame):
         Reset settings to defaults.
         """
 
-        # self.config = self._saved_config
         self._prot_nmea.set(self.__app.saved_config.get("nmeaprot_b", 1))
         self._prot_ubx.set(self.__app.saved_config.get("ubxprot_b", 1))
         self._prot_rtcm3.set(self.__app.saved_config.get("rtcmprot_b", 1))
@@ -519,12 +572,15 @@ class SettingsFrame(Frame):
             self.__app.saved_config.get("consoleformat_s", FORMAT_PARSED)
         )
         self.maptype.set(self.__app.saved_config.get("maptype_s", MAPTYPES[0]))
+        self.showtrack.set(self.__app.saved_config.get("showtrack_b", 0))
         self.mapzoom.set(self.__app.saved_config.get("mapzoom_n", 10))
         self.show_legend.set(self.__app.saved_config.get("legend_b", 1))
         self._show_unusedsat.set(self.__app.saved_config.get("unusedsat_b", 0))
         self._logformat.set(self.__app.saved_config.get("logformat_s", FORMAT_BINARY))
         self._datalog.set(self.__app.saved_config.get("datalog_b", 0))
+        self.logpath = self.__app.saved_config.get("logpath_s", HOME)
         self._record_track.set(self.__app.saved_config.get("recordtrack_b", 0))
+        self.trackpath = self.__app.saved_config.get("trackpath_s", HOME)
         self.clients = 0
 
     def _reset_frames(self):
@@ -569,12 +625,19 @@ class SettingsFrame(Frame):
         elif conntype == CONNECTED_SOCKET:
             frm = self.frm_socketclient
             if not frm.valid_settings():
-                self.__app.set_status("ERROR - invalid settings", "red")
+                self.__app.set_status("ERROR - invalid settings", ERRCOL)
                 return
             connstr = f"{frm.server.get()}:{frm.port.get()}"
             conndict = dict(conndict, **{"socket_settings": frm})
         elif conntype == CONNECTED_FILE:
-            self.infilepath = self.__app.file_handler.open_infile()
+            self.infilepath = self.__app.file_handler.open_file(
+                "datalog",
+                (
+                    ("datalog files", "*.log"),
+                    ("u-center logs", "*.ubx"),
+                    ("all files", "*.*"),
+                ),
+            )
             if self.infilepath is None:
                 return
             connstr = f"{self.infilepath}"
@@ -587,7 +650,7 @@ class SettingsFrame(Frame):
         else:
             return
 
-        self.__app.set_connection(connstr, "green")
+        self.__app.set_connection(connstr, OKCOL)
         self.__app.set_status("")
         self.__app.conn_status = conntype
         self._reset_frames()
@@ -600,6 +663,13 @@ class SettingsFrame(Frame):
 
         self.__app.start_dialog(DLGTUBX)
 
+    def _on_nmea_config(self, *args, **kwargs):  # pylint: disable=unused-argument
+        """
+        Open NMEA configuration dialog panel.
+        """
+
+        self.__app.start_dialog(DLGTNMEA)
+
     def _on_ntrip_config(self, *args, **kwargs):  # pylint: disable=unused-argument
         """
         Open NTRIP Client configuration dialog panel.
@@ -627,14 +697,14 @@ class SettingsFrame(Frame):
         """
 
         if self._datalog.get() == 1:
-            self.outfilepath = self.__app.file_handler.set_logfile_path()
-            if self.outfilepath is not None:
-                self.__app.set_status("Data logging enabled: " + self.outfilepath)
+            self.logpath = self.__app.file_handler.set_logfile_path(self.logpath)
+            if self.logpath is not None:
+                self.__app.set_status("Data logging enabled: " + self.logpath)
                 self.__app.file_handler.open_logfile()
             else:
+                self.logpath = ""
                 self._datalog.set(False)
         else:
-            self.outfilepath = None
             self._datalog.set(False)
             self.__app.file_handler.close_logfile()
             self.__app.set_status("Data logging disabled")
@@ -645,14 +715,14 @@ class SettingsFrame(Frame):
         """
 
         if self._record_track.get() == 1:
-            self._trackpath = self.__app.file_handler.set_trackfile_path()
-            if self._trackpath is not None:
-                self.__app.set_status("Track recording enabled: " + self._trackpath)
+            self.trackpath = self.__app.file_handler.set_trackfile_path(self.trackpath)
+            if self.trackpath is not None:
+                self.__app.set_status("Track recording enabled: " + self.trackpath)
                 self.__app.file_handler.open_trackfile()
             else:
+                self.trackpath = ""
                 self._record_track.set(False)
         else:
-            self._trackpath = None
             self._record_track.set(False)
             self.__app.file_handler.close_trackfile()
             self.__app.set_status("Track recording disabled")
@@ -761,13 +831,17 @@ class SettingsFrame(Frame):
                 "maxlines_n": self._maxlines.get(),
                 "consoleformat_s": self._console_format.get(),
                 "maptype_s": self.maptype.get(),
+                "showtrack_b": self.showtrack.get(),
                 "mapzoom_n": self.mapzoom.get(),
                 "legend_b": self.show_legend.get(),
                 "unusedsat_b": self._show_unusedsat.get(),
-                "logformat_s": self._logformat.get(),
                 "datalog_b": self._datalog.get(),
+                "logformat_s": self._logformat.get(),
+                "logpath_s": str(self.logpath),
                 "recordtrack_b": self._record_track.get(),
+                "trackpath_s": str(self.trackpath),
                 # serial port settings from frm_serial
+                "serialport_s": self.frm_serial.port,
                 "bpsrate_n": self.frm_serial.bpsrate,
                 "databits_n": self.frm_serial.databits,
                 "stopbits_f": self.frm_serial.stopbits,
@@ -899,9 +973,29 @@ class SettingsFrame(Frame):
                         0,
                     ),
                 ),
+                "scattersettings_d": self.__app.saved_config.get(
+                    "scattersettings_d",
+                    {
+                        "scatterautorange_b": 0,
+                        "scattercenter_s": "Average",
+                        "scatterinterval_n": 1,
+                        "scatterscale_n": 6,
+                        "scatterlat_f": 0.0,
+                        "scatterlon_f": 0.0,
+                    },
+                ),
+                "chartsettings_d": self.__app.saved_config.get(
+                    "chartsettings_d", {"numchn": 4, "timrng": 60}
+                ),
                 # Manually edited config settings
+                "checkforupdate_b": self.__app.saved_config.get("checkforupdate_b", 0),
                 "spartndecode_b": self.__app.saved_config.get("spartndecode_b", 0),
-                "spartnkey_s": self.__app.saved_config.get("spartnkey_s", ""),
+                "spartnkey_s": self.__app.saved_config.get(
+                    "spartnkey_s", SPARTN_DEFAULT_KEY
+                ),
+                "spartnbasedate_n": self.__app.saved_config.get(
+                    "spartnbasedate_n", SPARTN_BASEDATE_CURRENT
+                ),
                 "mapupdateinterval_n": self.__app.saved_config.get(
                     "mapupdateinterval_n", MAP_UPDATE_INTERVAL
                 ),
@@ -909,16 +1003,10 @@ class SettingsFrame(Frame):
                 "mqapikey_s": self.__app.saved_config.get("mqapikey_s", ""),
                 "colortags_l": self.__app.saved_config.get("colortags_l", []),
                 "ubxpresets_l": self.__app.saved_config.get("ubxpresets_l", []),
+                "nmeapresets_l": self.__app.saved_config.get("nmeapresets_l", []),
                 "usermaps_l": self.__app.saved_config.get("usermaps_l", []),
-                "scattermode_s": self.__app.saved_config.get("scattermode_s", "Planar"),
-                "scattercenter_s": self.__app.saved_config.get(
-                    "scattercenter_s", "Dynamic"
-                ),
-                "scatterscale_n": self.__app.saved_config.get("scatterscale_n", 6),
-                "scatterlat_f": self.__app.saved_config.get("scatterlat_f", 0.0),
-                "scatterlon_f": self.__app.saved_config.get("scatterlon_f", 0.0),
             }
             return config
         except (KeyError, ValueError, TypeError, TclError) as err:
-            self.__app.set_status(f"Error processing config data: {err}", BADCOL)
+            self.__app.set_status(f"Error processing config data: {err}", ERRCOL)
             return {}
