@@ -21,6 +21,7 @@ from pyubx2 import UBXMessage
 from pygpsclient.globals import (
     CONNECTED_SPARTNIP,
     CONNECTED_SPARTNLB,
+    ERRCOL,
     ICON_BLANK,
     ICON_CONFIRMED,
     ICON_EXIT,
@@ -86,14 +87,12 @@ class SPARTNConfigDialog(Toplevel):
 
         self._frm_container = Frame(self)
         self._frm_status = Frame(self._frm_container, borderwidth=2, relief="groove")
-        self._lbl_status = Label(
-            self._frm_status, textvariable=self._status, anchor="w"
-        )
+        self._lbl_status = Label(self._frm_status, textvariable=self._status, anchor=W)
         self._btn_exit = Button(
             self._frm_status,
             image=self._img_exit,
             width=55,
-            fg="red",
+            fg=ERRCOL,
             command=self.on_exit,
             font=self.__app.font_md,
         )
