@@ -13,32 +13,38 @@ Created on 23 Dec 2022
 :license: BSD 3-Clause
 """
 
-from math import ceil
-from tkinter import BOTH, YES, Canvas, Frame, font
+from tkinter import ALL, NW, Canvas, Checkbutton, E, Frame, IntVar, N, S, W
 
 from pyubx2 import UBXMessage
 
-from pygpsclient.globals import BGCOL, FGCOL, GNSS_LIST, SPECTRUMVIEW, WIDGETU2
-from pygpsclient.helpers import setubxrate
+from pygpsclient.globals import (
+    BGCOL,
+    FGCOL,
+    GNSS_LIST,
+    GRIDCOL,
+    PNTCOL,
+    SPECTRUMVIEW,
+    WIDGETU2,
+    AreaXY,
+)
+from pygpsclient.helpers import data2xy, fontheight, scale_font, setubxrate, xy2data
 from pygpsclient.strings import DLGENABLEMONSPAN, DLGNOMONSPAN, DLGWAITMONSPAN
 
 # Graph dimensions
 RESFONT = 24  # font size relative to widget size
 MINFONT = 7  # minimum font size
 OL_WID = 1
-MIN_DB = 20
-MAX_DB = 180
+MIN_DB = 0
+MAX_DB = 200
 MIN_HZ = 1130000000
 MAX_HZ = 1650000000
 TICK_DB = 20  # 20 dB divisions
 TICK_GHZ = 40000000  # 40 MHz divisions
-TICK_COL = "grey"
 RF_BANDS = {
     "B1": 1575420000,
     "B3": 1268520000,
     "B2": 1202025000,
     "B2a": 1176450000,
-    "SAR": 1544500000,
     "E6": 1278750000,
     "E5b": 1202025000,
     "E5a": 1176450000,
@@ -58,7 +64,18 @@ RF_LIST = {
     4: "dodgerblue",
     5: "darkorange",
 }
+RF_LIST_SNAPSHOT = {
+    0: "gray75",
+    1: "gray60",
+    2: "gray65",
+    3: "gray50",
+    4: "gray55",
+    5: "gray40",
+}
 ACTIVE = ""
+MODEINIT = "init"
+MODELIVE = "live"
+MODESNAP = "snap"
 
 
 class SpectrumviewFrame(Frame):
@@ -91,8 +108,11 @@ class SpectrumviewFrame(Frame):
         self._pending_confs = {}
         self._showrf = True
         self._chartpos = None
+        self._spectrum_snapshot = []
+        self._pgaoffset = IntVar()
+        self._font = self.__app.font_sm
+        self._fonth = fontheight(self._font)
         self._body()
-        self._set_fontsize()
         self._attach_events()
 
     def _body(self):
@@ -102,10 +122,21 @@ class SpectrumviewFrame(Frame):
 
         self.grid_columnconfigure(0, weight=1)
         self.grid_rowconfigure(0, weight=1)
+        self.grid_rowconfigure(1, weight=0)
         self.can_spectrumview = Canvas(
             self, width=self.width, height=self.height, bg=BGCOL
         )
-        self.can_spectrumview.pack(fill=BOTH, expand=YES)
+        self.chk_pgaoffset = Checkbutton(
+            self,
+            text="PGA Offset",
+            fg=PNTCOL,
+            bg=BGCOL,
+            variable=self._pgaoffset,
+            anchor=W,
+        )
+        # self.can_spectrumview.pack(fill=BOTH, expand=YES)
+        self.can_spectrumview.grid(column=0, row=0, columnspan=3, sticky=(N, S, E, W))
+        self.chk_pgaoffset.grid(column=0, row=1, sticky=(W, E))
 
     def _attach_events(self):
         """
@@ -114,22 +145,33 @@ class SpectrumviewFrame(Frame):
 
         self.bind("<Configure>", self._on_resize)
         self.can_spectrumview.bind("<Button-1>", self._on_click)
-        self.can_spectrumview.bind("<Double-Button-1>", self._toggle_rf)
+        self.can_spectrumview.bind("<Double-Button-1>", self._on_toggle_rf)
+        self.can_spectrumview.bind("<Button-2>", self._on_snapshot)
+        self.can_spectrumview.bind("<Double-Button-2>", self._on_clear_snapshot)
 
     def init_frame(self):
         """
         Initialise spectrum chart.
         """
 
-        self.can_spectrumview.delete("all")
+        offset = self._fonth + 4
+        w = self.width - offset * 2
+        h = self.height - offset
+        bounds = AreaXY(self._minhz, self._mindb, self._maxhz, self._maxdb)
+        self.can_spectrumview.delete(ALL)
 
         # plot y (dB) axis grid
         i = 0
         for db in range(self._mindb, self._maxdb, TICK_DB):
-            x1, y1 = self._get_point(self._minhz, db)
-            x2, y2 = self._get_point(self._maxhz, db)
+            x1, y1 = data2xy(w, h, bounds, self._minhz, db, offset)
+            x2, y2 = data2xy(w, h, bounds, self._maxhz, db, offset)
             self.can_spectrumview.create_line(
-                x1, y1, x2 + 1, y1, fill=TICK_COL if i else FGCOL
+                x1,
+                y1,
+                x2 + 1,
+                y1,
+                fill=GRIDCOL if i else FGCOL,
+                tags=MODEINIT,
             )
             self.can_spectrumview.create_text(
                 x1,
@@ -138,17 +180,23 @@ class SpectrumviewFrame(Frame):
                 angle=90,
                 fill=FGCOL,
                 font=self._font,
-                anchor="s",
+                anchor=S,
+                tags=MODEINIT,
             )
             i += 1
 
         # plot x (Hz) axis grid
         i = 0
         for hz in range(self._minhz, self._maxhz, TICK_GHZ):
-            x1, y1 = self._get_point(hz, self._mindb)
-            x2, y2 = self._get_point(hz, self._maxdb)
+            x1, y1 = data2xy(w, h, bounds, hz, self._mindb, offset)
+            x2, y2 = data2xy(w, h, bounds, hz, self._maxdb, offset)
             self.can_spectrumview.create_line(
-                x1, y1 - 1, x1, y2, fill=TICK_COL if i else FGCOL
+                x1,
+                y1 - 1,
+                x1,
+                y2,
+                fill=GRIDCOL if i else FGCOL,
+                tags=MODEINIT,
             )
             self.can_spectrumview.create_text(
                 x1,
@@ -156,11 +204,12 @@ class SpectrumviewFrame(Frame):
                 text=f"{hz / 1e9:.2f}",  # GHz
                 fill=FGCOL,
                 font=self._font,
-                anchor="n",
+                anchor=N,
+                tags=MODEINIT,
             )
             i += 1
 
-        x, y = self._get_point(self._maxhz, self._mindb)
+        x, y = data2xy(w, h, bounds, self._maxhz, self._mindb, offset)
         self.can_spectrumview.create_text(
             x,
             y,
@@ -168,8 +217,9 @@ class SpectrumviewFrame(Frame):
             fill=FGCOL,
             font=self._font,
             anchor="se",
+            tags=MODEINIT,
         )
-        x, y = self._get_point(self._minhz + self._fonth, self._maxdb - 5)
+        x, y = data2xy(w, h, bounds, self._minhz + self._fonth, self._maxdb - 5, offset)
         self.can_spectrumview.create_text(
             x,
             y,
@@ -178,6 +228,7 @@ class SpectrumviewFrame(Frame):
             angle=90,
             font=self._font,
             anchor="ne",
+            tags=MODEINIT,
         )
 
         # display 'enable MON-SPAN' warning
@@ -186,6 +237,7 @@ class SpectrumviewFrame(Frame):
             self.height / 2,
             text=self._monspan_status,
             fill="orange",
+            tags=MODEINIT,
         )
 
     def reset(self):
@@ -195,7 +247,8 @@ class SpectrumviewFrame(Frame):
 
         self.__app.gnss_status.spectrum_data = []
         self._chartpos = None
-        self.can_spectrumview.delete("all")
+        self._pgaoffset.set(0)
+        self.can_spectrumview.delete(ALL)
         self.update_frame()
 
     def enable_messages(self, status: bool):
@@ -239,7 +292,7 @@ class SpectrumviewFrame(Frame):
                 h / 2,
                 text=DLGNOMONSPAN,
                 fill="orange",
-                anchor="s",
+                anchor=S,
             )
             self._pending_confs.pop("ACK-NAK")
             self._monspan_status = DLGNOMONSPAN
@@ -260,52 +313,62 @@ class SpectrumviewFrame(Frame):
         if len(rfblocks) == 0:
             return
         self._monspan_status = ACTIVE
+        self._update_plot(rfblocks)
 
-        specxy, self._minhz, self._maxhz, self._mindb, self._maxdb = self._get_limits(
-            rfblocks
-        )
-        self.init_frame()
+        if self._spectrum_snapshot != []:
+            self._update_plot(self._spectrum_snapshot, MODESNAP, RF_LIST_SNAPSHOT)
 
-        # plot frequency band markers
-        if self._showrf:
-            for nam, frq in RF_BANDS.items():
-                if self._minhz < frq < self._maxhz:
-                    x1, y1 = self._get_point(frq, self._maxdb)
-                    x2, y2 = self._get_point(frq, self._mindb)
-                    yoff, col = {
-                        "L": (self._fonth, GNSS_LIST[0][1]),  # GPS
-                        "G": (self._fonth * 2, GNSS_LIST[6][1]),  # GLONASS
-                        "E": (self._fonth * 3, GNSS_LIST[2][1]),  # Galileo
-                        "S": (self._fonth * 3, GNSS_LIST[2][1]),  # Galileo SAR
-                        "B": (self._fonth * 4, GNSS_LIST[3][1]),  # Beidou
-                    }[nam[0:1]]
-                    if nam not in (
-                        "E1",
-                        "E5a",
-                        "E5b",
-                        "B2a",
-                        "B2",
-                        "B1",
-                    ):  # same freq as other bands
-                        self.can_spectrumview.create_line(
-                            x1, y1, x1, y2, fill=col, dash=(5, 2), width=OL_WID
-                        )
-                    self.can_spectrumview.create_text(
-                        x2 + 2,
-                        y2 - yoff - 1,
-                        text=nam,
-                        fill=col,
-                        anchor="nw",
-                        font=self._font,
-                    )
+    def _update_plot(self, rfblocks: list, mode: str = MODELIVE, colors: dict = None):
+        """
+        Update spectrum plot with live or snapshot rf block data.
+
+        :param list rfblocks: array of spectrum rf blocks
+        :param dict colors: dictionary of color for each rf block
+        :param str mode: plot mode ("live" or "snap"shot)
+        """
+
+        offset = self._fonth + 4
+        w = self.width - offset * 2
+        h = self.height - offset
+
+        if colors is None:
+            colors = RF_LIST
+
+        self._mindb = MIN_DB
+        self._maxdb = MAX_DB
+        if self._pgaoffset.get():
+            self._maxdb += 40
+
+        specxy, self._minhz, self._maxhz = self._get_limits(rfblocks)
+        bounds = AreaXY(self._minhz, self._mindb, self._maxhz, self._maxdb)
+        if mode == MODESNAP:
+            self.can_spectrumview.delete(MODESNAP)
+        else:
+            self.init_frame()
+            # plot frequency band markers
+            if self._showrf:
+                self._plot_rf_markers(w, h, bounds, offset, mode)
 
         # for each RF block in MON-SPAN message
         for i, rfblock in enumerate(specxy):
-            col = RF_LIST[i % len(RF_LIST)]
+            col = colors[i % len(colors)]
 
             # draw legend for this RF block
             x1, y1 = (50 * (i + 1) + (i * self._fonth), self._fonth * 2)
             x2, y2 = (50 + 50 * (i + 1) + (i * self._fonth), self._fonth * 2)
+            if mode == MODESNAP:
+                y1 += self._fonth
+                y2 += self._fonth
+            else:
+                self.can_spectrumview.create_text(
+                    (x1 + x2) / 2,
+                    y1,
+                    text=f"RF {i + 1}",
+                    fill=FGCOL,
+                    font=self._font,
+                    anchor=S,
+                    tags=mode,
+                )
             self.can_spectrumview.create_line(
                 x1,
                 y1,
@@ -313,14 +376,7 @@ class SpectrumviewFrame(Frame):
                 y1,
                 fill=col,
                 width=OL_WID,
-            )
-            self.can_spectrumview.create_text(
-                (x1 + x2) / 2,
-                y1,
-                text=f"RF {i + 1}",
-                fill=FGCOL,
-                font=self._font,
-                anchor="s",
+                tags=mode,
             )
 
             # plot spectrum for this RF block
@@ -328,10 +384,16 @@ class SpectrumviewFrame(Frame):
             y2 = self.height - 5
             for n, (hz, db) in enumerate(rfblock):
                 x1, y1 = x2, y2
-                x2, y2 = self._get_point(hz, db)
+                x2, y2 = data2xy(w, h, bounds, hz, db, offset)
                 if n:
                     self.can_spectrumview.create_line(
-                        x1, y1, x2, y2, fill=col, width=OL_WID
+                        x1,
+                        y1,
+                        x2,
+                        y2,
+                        fill=col,
+                        width=OL_WID,
+                        tags=mode,
                     )
 
         # display any flagged chart position
@@ -344,53 +406,58 @@ class SpectrumviewFrame(Frame):
                 fill=FGCOL,
                 font=self._font,
                 anchor="center",
+                tags=mode,
             )
 
-        # self.can_spectrumview.update_idletasks()
-
-    def _get_point(self, hz: float, db: float) -> tuple:
+    def _plot_rf_markers(self, w: int, h: int, bounds: AreaXY, offset: int, mode: int):
         """
-        Convert (hz,db) values to canvas pixel coordinates (x,y).
+        Plot RF band markers
 
-        :param hz float: hz (x) value
-        :param db float: db (y) value
-        :return: (x, y) coordinates
-        :rtype: tuple
+        :param int w: plots width
+        :param int h: plot height
+        :param AreaXY bounds: data bounds
+        :param offset: _description_
         """
+        # pylint: disable=too-many-arguments, too-many-positional-arguments
 
-        offset = self._fonth + 4
-        val_db = db - self._mindb
-        range_db = self._maxdb - self._mindb
-        range_x = self.width - offset * 2
-        val_hz = hz - self._minhz
-        range_hz = self._maxhz - self._minhz
-        range_y = self.height - self._fonth
-
-        x = offset + (val_hz * range_x / range_hz)
-        y = self.height - offset - (val_db * range_y / range_db)
-        return (int(x), int(y))
-
-    def _get_hzdb(self, x: int, y: int) -> tuple:
-        """
-        Get frequency & level corresponding to cursor x,y position.
-
-        :param x int: cursor x position
-        :param y int: cursor y position
-        :return: (Ghz, dB) values
-        :rtype: tuple
-        """
-
-        offset = self._fonth + 4
-        val_x = x - offset
-        range_x = self.width - offset * 2
-        range_db = self._maxdb - self._mindb
-        val_y = y - self.height + offset
-        range_y = self._fonth - self.height
-        range_hz = self._maxhz - self._minhz
-
-        hz = self._minhz + (val_x * range_hz / range_x)
-        db = self._mindb + (val_y * range_db / range_y)
-        return (hz / 1e9, db)
+        for nam, frq in RF_BANDS.items():
+            if self._minhz < frq < self._maxhz:
+                x1, y1 = data2xy(w, h, bounds, frq, self._maxdb, offset)
+                x2, y2 = data2xy(w, h, bounds, frq, self._mindb, offset)
+                yoff, col = {
+                    "L": (self._fonth, GNSS_LIST[0][1]),  # GPS
+                    "G": (self._fonth * 2, GNSS_LIST[6][1]),  # GLONASS
+                    "E": (self._fonth * 3, GNSS_LIST[2][1]),  # Galileo
+                    "S": (self._fonth * 3, GNSS_LIST[2][1]),  # Galileo SAR
+                    "B": (self._fonth * 4, GNSS_LIST[3][1]),  # Beidou
+                }[nam[0:1]]
+                if nam not in (
+                    "E1",
+                    "E5a",
+                    "E5b",
+                    "B2a",
+                    "B2",
+                    "B1",
+                ):  # same freq as other bands
+                    self.can_spectrumview.create_line(
+                        x1,
+                        y1,
+                        x1,
+                        y2,
+                        fill=col,
+                        dash=(5, 2),
+                        width=OL_WID,
+                        tags=mode,
+                    )
+                self.can_spectrumview.create_text(
+                    x2 + 2,
+                    y2 - yoff - 1,
+                    text=nam,
+                    fill=col,
+                    anchor=NW,
+                    font=self._font,
+                    tags=mode,
+                )
 
     def _get_limits(self, rfblocks: list) -> tuple:
         """
@@ -405,20 +472,17 @@ class SpectrumviewFrame(Frame):
 
         minhz = 999 * 1e9
         maxhz = 0
-        mindb = 0
-        maxdb = 0
         specxy = []
 
         # for each RF block in MON-SPAN message
         for i, rfblock in enumerate(rfblocks):
-            (spec, spn, res, ctr, _) = rfblock
+            (spec, spn, res, ctr, pga) = rfblock
             minhz = int(min(minhz, ctr - res * (spn / res) / 2))
             maxhz = int(max(maxhz, ctr + res * (spn / res) / 2))
             spanhz = []
             for i, db in enumerate(spec):
-                # db -= pga  # compensate for programmable gain
-                mindb = min(mindb, db)
-                maxdb = max(maxdb, db)
+                if self._pgaoffset.get():
+                    db += pga  # compensate for programmable gain
                 hz = int(minhz + (res * i))
                 spanhz.append((hz, db))
             specxy.append(spanhz)
@@ -426,11 +490,44 @@ class SpectrumviewFrame(Frame):
         return (
             specxy,
             int(minhz - TICK_GHZ / 2),
-            maxhz,
-            MIN_DB,
-            (ceil((maxdb + 10) / 10) * 10) + TICK_DB,
+            int(maxhz + TICK_GHZ / 2),
         )
 
+    def _on_click(self, event):
+        """
+        Save flagged chart position.
+        """
+
+        offset = self._fonth + 4
+        w = self.width - offset * 2
+        h = self.height - offset
+        bounds = AreaXY(self._minhz, self._mindb, self._maxhz, self._maxdb)
+        hz, db = xy2data(w, h, bounds, event.x - offset, event.y)
+        self._chartpos = (event.x, event.y, hz / 1e9, db)
+
+    def _on_toggle_rf(self, event):  # pylint: disable=unused-argument
+        """
+        Toggle RF band markers on/off.
+        """
+
+        self._showrf = not self._showrf
+        self._chartpos = None
+
+    def _on_snapshot(self, event):  # pylint: disable=unused-argument
+        """
+        Capture snapshot of current spectrum.
+        """
+
+        self._spectrum_snapshot = self.__app.gnss_status.spectrum_data
+
+    def _on_clear_snapshot(self, event):  # pylint: disable=unused-argument
+        """
+        Clear snapshot of current spectrum.
+        """
+
+        self._spectrum_snapshot = []
+        self.can_spectrumview.delete("snap")
+
     def _on_resize(self, event):  # pylint: disable=unused-argument
         """
         Resize frame.
@@ -439,22 +536,7 @@ class SpectrumviewFrame(Frame):
         """
 
         self.width, self.height = self.get_size()
-        self._chartpos = None
-
-    def _on_click(self, event):
-        """
-        Save flagged chart position.
-        """
-
-        hz, db = self._get_hzdb(event.x, event.y)
-        self._chartpos = (event.x, event.y, hz, db)
-
-    def _toggle_rf(self, event):  # pylint: disable=unused-argument
-        """
-        Toggle RF band markers on/off.
-        """
-
-        self._showrf = not self._showrf
+        self._font, self._fonth = scale_font(self.width, 8, 25, 20)
         self._chartpos = None
 
     def get_size(self):
@@ -466,16 +548,4 @@ class SpectrumviewFrame(Frame):
         """
 
         self.update_idletasks()  # Make sure we know about any resizing
-        width = self.can_spectrumview.winfo_width()
-        height = self.can_spectrumview.winfo_height()
-        self._set_fontsize()
-        return (width, height)
-
-    def _set_fontsize(self):
-        """
-        Set font size and line spacing
-        """
-
-        dim = min(self.width, self.height)
-        self._font = font.Font(size=max(int(dim * RESFONT / 1000), MINFONT))
-        self._fonth = self._font.metrics("linespace")
+        return self.can_spectrumview.winfo_width(), self.can_spectrumview.winfo_height()
