@@ -35,7 +35,6 @@ from queue import Empty
 from threading import Event, Thread
 
 from certifi import where as findcacerts
-from pygnssutils import UBXSimulator
 from pynmeagps import NMEAMessageError, NMEAParseError
 from pyrtcm import RTCMMessageError, RTCMParseError
 from pyubx2 import (
@@ -47,6 +46,7 @@ from pyubx2 import (
     UBXParseError,
     UBXReader,
 )
+from pyubxutils import UBXSimulator
 from serial import Serial, SerialException, SerialTimeoutException
 
 from pygpsclient.globals import (
@@ -55,6 +55,7 @@ from pygpsclient.globals import (
     CONNECTED_SIMULATOR,
     CONNECTED_SOCKET,
     DEFAULT_BUFSIZE,
+    ERRCOL,
     FILEREAD_INTERVAL,
     UBXSIMULATOR,
 )
@@ -216,7 +217,7 @@ class StreamHandler:
                 stopevent.set()
                 self.__master.event_generate(settings["error_event"])
                 if hasattr(caller, "set_status"):
-                    caller.set_status(str(err), "red")
+                    caller.set_status(str(err), ERRCOL)
 
     def _readloop(
         self,
