@@ -15,7 +15,7 @@ Created on 12 Sep 2020
 from pygpsclient.globals import GITHUB_URL, MAPAPI_URL
 
 TITLE = "PyGPSClient"
-COPYRIGHT = "\u00A9 2020 SEMU Consulting\nBSD 3-Clause License"
+COPYRIGHT = "\u00a9 2020 SEMU Consulting\nBSD 3-Clause License"
 COPYRIGHTTXT = f"{COPYRIGHT}. All Rights Reserved"
 EPILOG = f"{COPYRIGHT} - {GITHUB_URL}"
 
@@ -39,6 +39,7 @@ CONFIGERR = "Invalid configuration data"
 CONFIGOK = "{} command accepted"
 CONFIGRXM = "{} polled, {} key(s) loaded"
 CONFIGTITLE = "Config File"
+CONFIRM = "CONFIRM"
 ENDOFFILE = "End of file reached"
 FILEOPENERROR = "Error opening file {}"
 INACTIVE_TIMEOUT = "Inactivity timeout"
@@ -57,7 +58,7 @@ NOWEBMAPFIX = NOWEBMAP + "\nNo satellite fix."
 NOWEBMAPHTTP = NOWEBMAP + "\nBad HTTP response: {}.\nCheck MQAPIKEY.\n"
 NOWEBMAPKEY = NOWEBMAP + f"\nMQAPIKEY not found or invalid.\n\n{MAPAPI_URL}"
 NULLSEND = "Nothing to send"
-OUTOFBOUNDS = "No custom map available"
+OUTOFBOUNDS = "No custom map available for location"
 OPENFILEERROR = "ERROR! File could not be opened"
 READTITLE = "Select File"
 SAVECONFIGBAD = "Configuration not saved {}"
@@ -73,6 +74,7 @@ VERCHECK = f"Newer version of {TITLE} available:"
 WAITNMEADATA = "Waiting for data..."
 WAITUBXDATA = "Waiting for data..."
 NOWDGSWARN = "WARNING! No widgets are enabled in config file {} - display will be blank"
+HALTTAGWARN = "HALTED ON USER TAG MATCH: {}"
 
 # Menu text
 MENUABOUT = "About"
@@ -96,7 +98,8 @@ BTNSAVE = "Save"
 
 # Label text
 LBLACCURACY = "Accuracy (cm)"
-LBLCFGGENERIC = "CFG-* Generic Configuration"
+LBLCFGGENERIC = "UBX Legacy Command Configuration"
+LBLCFGGENERICNMEA = "NMEA Command Configuration"
 LBLCFGMSG = "CFG-MSG Message Rate Configuration"
 LBLCFGPRT = "CFG-PRT Protocol Configuration"
 LBLCFGRATE = "CFG-RATE Navigation Solution Rate Configuration"
@@ -111,6 +114,8 @@ LBLGGAFIXED = "Fixed Reference"
 LBLGGALIVE = "Receiver"
 LBLJSONLOAD = "Load Keys From JSON"
 LBLLANIP = "LAN IP"
+LBLSHOWTRACK = "Show Track"
+LBLNMEACONFIG = "NMEA Config"
 LBLNTRIPCONFIG = "NTRIP Client"
 LBLNTRIPGGAINT = "GGA Interval s"
 LBLNTRIPMOUNT = "Mountpoint"
@@ -120,7 +125,8 @@ LBLNTRIPSERVER = "Server"
 LBLNTRIPSTR = "Sourcetable"
 LBLNTRIPUSER = "User"
 LBLNTRIPVERSION = "Version"
-LBLPRESET = "Preset UBX Configuration Commands"
+LBLNMEAPRESET = "Preset NMEA Configuration Commands"
+LBLUBXPRESET = "Preset UBX Configuration Commands"
 LBLPROTDISP = "Protocols Shown"
 LBLPUBLICIP = "Public IP"
 LBLSERVERHOST = "Host IP"
@@ -143,6 +149,7 @@ LBLSTREAM = "Stream\nfrom file"
 LBLTRACKRECORD = "GPX Track"
 LBLUBXCONFIG = "UBX Config"
 LBLUDPORT = "USER-DEFINED PORT"
+LBLSPORT = "SAVED PORT"
 LBLDISNMEA = "Disable NMEA"
 
 # Dialog text
@@ -161,14 +168,11 @@ DLGJSONOK = "Keys loaded from {}"
 DLGNOMONSPAN = "This receiver does not appear to\nsupport the MON-SPAN messages"
 DLGNOMONSYS = "This receiver does not appear to support\nthe MON-SYS/COMMS messages"
 DLGNTRIPCONFIG = "NTRIP Client Configuration"
-DLGRESET = "Confirm Reset"
-DLGRESETCONFIRM = (
-    "Are you sure you want to reset the\ncurrent configuration to the\nfactory default?"
-)
-DLGSAVE = "Confirm Save"
-DLGSAVECONFIRM = "Are you sure you want to save\nthe current configuration?"
+DLGACTION = "Confirm Command"
+DLGACTIONCONFIRM = "Are you sure?"
 DLGSPARTNCONFIG = "SPARTN Client Configuration"
 DLGSPARTNWARN = "WARNING! Disconnect from {} client before using {} client"
+DLGNMEACONFIG = "NMEA Configuration"
 DLGUBXCONFIG = "UBX Configuration"
 DLGWAITMONSPAN = "Waiting for MON-SPAN message..."
 DLGWAITMONSYS = "Waiting for MON-SYS/COMMS messages..."
@@ -177,6 +181,7 @@ DLGTABOUT = "About"
 DLGTGPX = "GPX Track Viewer"
 DLGTNTRIP = "NTRIP Configuration"
 DLGTSPARTN = "SPARTN Configuration"
+DLGTNMEA = "NMEA Configuration"
 DLGTUBX = "UBX Configuration"
 DLGTIMPORTMAP = "Import Custom Map"
 
