@@ -36,11 +36,14 @@ from pyubx2 import UBX_CONFIG_DATABASE, UBXMessage
 from pyubx2.ubxhelpers import attsiz, atttyp, cfgname2key
 
 from pygpsclient.globals import (
+    ERRCOL,
     ICON_CONFIRMED,
     ICON_PENDING,
     ICON_SEND,
     ICON_WARNING,
+    OKCOL,
     READONLY,
+    RPTDELAY,
     UBX_CFGVAL,
 )
 
@@ -103,14 +106,14 @@ class UBX_CFGVAL_Frame(Frame):
         """
 
         self._lbl_configdb = Label(
-            self, text="CFG-VALSET/DEL/GET Configuration Interface", anchor="w"
+            self, text="CFG-VALSET/DEL/GET Configuration Interface", anchor=W
         )
-        self._lbl_cat = Label(self, text="Category", anchor="w")
+        self._lbl_cat = Label(self, text="Category", anchor=W)
         self._lbx_cat = Listbox(
             self,
             border=2,
             relief="sunken",
-            height=5,
+            height=12,
             justify=LEFT,
             exportselection=False,
         )
@@ -120,12 +123,12 @@ class UBX_CFGVAL_Frame(Frame):
         self._lbx_cat.config(xscrollcommand=self._scr_cath.set)
         self._scr_catv.config(command=self._lbx_cat.yview)
         self._scr_cath.config(command=self._lbx_cat.xview)
-        self._lbl_parm = Label(self, text="Keyname", anchor="w")
+        self._lbl_parm = Label(self, text="Keyname", anchor=W)
         self._lbx_parm = Listbox(
             self,
             border=2,
             relief="sunken",
-            height=5,
+            height=12,
             justify=LEFT,
             exportselection=False,
         )
@@ -169,6 +172,8 @@ class UBX_CFGVAL_Frame(Frame):
             wrap=True,
             width=8,
             state=READONLY,
+            repeatdelay=RPTDELAY,
+            repeatinterval=RPTDELAY,
         )
         self._lbl_val = Label(self, text="Value")
         self._ent_val = Entry(
@@ -372,7 +377,7 @@ class UBX_CFGVAL_Frame(Frame):
                     "INVALID ENTRY - must conform to parameter "
                     f"type {att} ({typ}) and size {atts} bytes"
                 ),
-                "red",
+                ERRCOL,
             )
 
         return valid_entry
@@ -439,12 +444,12 @@ class UBX_CFGVAL_Frame(Frame):
                 if isinstance(val, bytes):
                     val = val.hex()
                 self._cfgval.set(val)
-            self.__container.set_status("CFG-VALGET GET message received", "green")
+            self.__container.set_status("CFG-VALGET GET message received", OKCOL)
 
         elif msg.identity == "ACK-ACK":
             self._lbl_send_command.config(image=self._img_confirmed)
-            self.__container.set_status("CFG-VAL command acknowledged", "green")
+            self.__container.set_status("CFG-VAL command acknowledged", OKCOL)
 
         elif msg.identity == "ACK-NAK":
             self._lbl_send_command.config(image=self._img_warn)
-            self.__container.set_status("CFG-VAL command rejected", "red")
+            self.__container.set_status("CFG-VAL command rejected", ERRCOL)
