@@ -28,11 +28,13 @@ from tkinter import Button, E, Frame, Label, N, S, StringVar, Toplevel, W
 from PIL import Image, ImageTk
 from pyubx2 import UBXMessage
 
+from pygpsclient.dynamic_config_frame import Dynamic_Config_Frame
 from pygpsclient.globals import (
     CONNECTED,
     CONNECTED_SIMULATOR,
     CONNECTED_SOCKET,
     ENABLE_CFG_OTHER,
+    ERRCOL,
     ICON_EXIT,
     POPUP_TRANSIENT,
     SAVED_CONFIG,
@@ -46,10 +48,9 @@ from pygpsclient.globals import (
     UBX_MONVER,
     UBX_PRESET,
 )
+from pygpsclient.hardware_info_frame import Hardware_Info_Frame
 from pygpsclient.strings import DLGTUBX, DLGUBXCONFIG
 from pygpsclient.ubx_cfgval_frame import UBX_CFGVAL_Frame
-from pygpsclient.ubx_dynamic_frame import UBX_Dynamic_Frame
-from pygpsclient.ubx_info_frame import UBX_INFO_Frame
 from pygpsclient.ubx_msgrate_frame import UBX_MSGRATE_Frame
 from pygpsclient.ubx_port_frame import UBX_PORT_Frame
 from pygpsclient.ubx_preset_frame import UBX_PRESET_Frame
@@ -99,20 +100,18 @@ class UBXConfigDialog(Toplevel):
 
         self._frm_container = Frame(self, borderwidth=2, relief="groove")
         self._frm_status = Frame(self._frm_container, borderwidth=2, relief="groove")
-        self._lbl_status = Label(
-            self._frm_status, textvariable=self._status, anchor="w"
-        )
+        self._lbl_status = Label(self._frm_status, textvariable=self._status, anchor=W)
         self._btn_exit = Button(
             self._frm_status,
             image=self._img_exit,
             width=50,
-            fg="red",
+            fg=ERRCOL,
             command=self.on_exit,
             font=self.__app.font_md,
         )
         # add configuration widgets
-        self._frm_device_info = UBX_INFO_Frame(
-            self.__app, self, borderwidth=2, relief="groove"
+        self._frm_device_info = Hardware_Info_Frame(
+            self.__app, self, borderwidth=2, relief="groove", protocol="UBX"
         )
         self._frm_recorder = UBX_Recorder_Frame(
             self.__app, self, borderwidth=2, relief="groove"
@@ -126,10 +125,9 @@ class UBXConfigDialog(Toplevel):
         self._frm_config_msg = UBX_MSGRATE_Frame(
             self.__app, self, borderwidth=2, relief="groove"
         )
-        if ENABLE_CFG_OTHER:
-            self._frm_config_dynamic = UBX_Dynamic_Frame(
-                self.__app, self, borderwidth=2, relief="groove"
-            )
+        self._frm_config_dynamic = Dynamic_Config_Frame(
+            self.__app, self, borderwidth=2, relief="groove", protocol="UBX"
+        )
         self._frm_configdb = UBX_CFGVAL_Frame(
             self.__app, self, borderwidth=2, relief="groove"
         )
@@ -243,7 +241,7 @@ class UBXConfigDialog(Toplevel):
             CONNECTED_SOCKET,
             CONNECTED_SIMULATOR,
         ):
-            self.set_status("Device not connected", "red")
+            self.set_status("Device not connected", ERRCOL)
 
     def set_pending(self, msgid: int, ubxfrm: int):
         """
@@ -317,7 +315,7 @@ class UBXConfigDialog(Toplevel):
         """
 
         self.__master.update_idletasks()  # Make sure we know about any resizing
-        return (self.winfo_width(), self.winfo_height())
+        return self.winfo_width(), self.winfo_height()
 
     @property
     def container(self):
