@@ -33,11 +33,14 @@ from pyubx2 import POLL, SET, UBX_MSGIDS, UBXMessage
 from pyubx2.ubxhelpers import key_from_val
 
 from pygpsclient.globals import (
+    ERRCOL,
     ICON_CONFIRMED,
     ICON_PENDING,
     ICON_SEND,
     ICON_WARNING,
+    OKCOL,
     READONLY,
+    RPTDELAY,
     UBX_CFGMSG,
 )
 from pygpsclient.strings import LBLCFGMSG
@@ -86,7 +89,7 @@ class UBX_MSGRATE_Frame(Frame):
         """
 
         MAX_RATE = 0xFF
-        self._lbl_cfg_msg = Label(self, text=LBLCFGMSG, anchor="w")
+        self._lbl_cfg_msg = Label(self, text=LBLCFGMSG, anchor=W)
         self._lbx_cfg_msg = Listbox(
             self,
             border=2,
@@ -106,6 +109,8 @@ class UBX_MSGRATE_Frame(Frame):
             to=MAX_RATE,
             textvariable=self._ddc_rate,
             state=READONLY,
+            repeatdelay=RPTDELAY,
+            repeatinterval=RPTDELAY,
         )
         self._lbl_uart1 = Label(self, text="UART1")
         self._spn_uart1 = Spinbox(
@@ -115,6 +120,8 @@ class UBX_MSGRATE_Frame(Frame):
             to=MAX_RATE,
             textvariable=self._uart1_rate,
             state=READONLY,
+            repeatdelay=RPTDELAY,
+            repeatinterval=RPTDELAY,
         )
         self._lbl_uart2 = Label(self, text="UART2")
         self._spn_uart2 = Spinbox(
@@ -124,6 +131,8 @@ class UBX_MSGRATE_Frame(Frame):
             to=MAX_RATE,
             textvariable=self._uart2_rate,
             state=READONLY,
+            repeatdelay=RPTDELAY,
+            repeatinterval=RPTDELAY,
         )
         self._lbl_usb = Label(self, text="USB")
         self._spn_usb = Spinbox(
@@ -133,6 +142,8 @@ class UBX_MSGRATE_Frame(Frame):
             to=MAX_RATE,
             textvariable=self._usb_rate,
             state=READONLY,
+            repeatdelay=RPTDELAY,
+            repeatinterval=RPTDELAY,
         )
         self._lbl_spi = Label(self, text="SPI")
         self._spn_spi = Spinbox(
@@ -142,13 +153,15 @@ class UBX_MSGRATE_Frame(Frame):
             to=MAX_RATE,
             textvariable=self._spi_rate,
             state=READONLY,
+            repeatdelay=RPTDELAY,
+            repeatinterval=RPTDELAY,
         )
         self._lbl_send_command = Label(self)
         self._btn_send_command = Button(
             self,
             image=self._img_send,
             width=50,
-            fg="green",
+            fg=OKCOL,
             command=self._on_send_cfg_msg,
             font=self.__app.font_md,
         )
@@ -213,7 +226,7 @@ class UBX_MSGRATE_Frame(Frame):
         """
 
         if msg.identity == "CFG-MSG":
-            self.__container.set_status("CFG-MSG GET message received", "green")
+            self.__container.set_status("CFG-MSG GET message received", OKCOL)
             self._ddc_rate.set(msg.rateDDC)
             self._uart1_rate.set(msg.rateUART1)
             self._uart2_rate.set(msg.rateUART2)
@@ -222,7 +235,7 @@ class UBX_MSGRATE_Frame(Frame):
             self._lbl_send_command.config(image=self._img_confirmed)
 
         elif msg.identity == "ACK-NAK":
-            self.__container.set_status("CFG-MSG POLL message rejected", "red")
+            self.__container.set_status("CFG-MSG POLL message rejected", ERRCOL)
             self._lbl_send_command.config(image=self._img_warn)
 
     def _on_select_cfg_msg(self, *args, **kwargs):  # pylint: disable=unused-argument
@@ -264,7 +277,7 @@ class UBX_MSGRATE_Frame(Frame):
         )
         self.__container.send_command(msg)
         self._lbl_send_command.config(image=self._img_pending)
-        self.__container.set_status("CFG-MSG SET message sent", "green")
+        self.__container.set_status("CFG-MSG SET message sent", OKCOL)
         for msgid in ("ACK-ACK", "ACK-NAK"):
             self.__container.set_pending(msgid, UBX_CFGMSG)
 
