@@ -15,18 +15,21 @@ from tkinter import Button, Checkbutton, E, Frame, IntVar, Label, Spinbox, Strin
 from PIL import Image, ImageTk
 from pyubx2 import POLL, SET, UBXMessage
 
-from .globals import (
+from pygpsclient.globals import (
     BPSRATES,
     CONNECTED,
+    ERRCOL,
     ICON_CONFIRMED,
     ICON_PENDING,
     ICON_SEND,
     ICON_WARNING,
+    OKCOL,
     PORTIDS,
     READONLY,
+    RPTDELAY,
     UBX_CFGPRT,
 )
-from .strings import LBLCFGPRT
+from pygpsclient.strings import LBLCFGPRT
 
 
 class UBX_PORT_Frame(Frame):
@@ -75,7 +78,7 @@ class UBX_PORT_Frame(Frame):
         Set up frame and widgets.
         """
 
-        self._lbl_cfg_port = Label(self, text=LBLCFGPRT, anchor="w")
+        self._lbl_cfg_port = Label(self, text=LBLCFGPRT, anchor=W)
         self._lbl_ubx_portid = Label(self, text="Port ID")
         self._spn_ubx_portid = Spinbox(
             self,
@@ -83,6 +86,8 @@ class UBX_PORT_Frame(Frame):
             width=8,
             state=READONLY,
             wrap=True,
+            repeatdelay=RPTDELAY,
+            repeatinterval=RPTDELAY,
             textvariable=self._portid,
             command=lambda: self._on_select_portid(),  # pylint: disable=unnecessary-lambda
         )
@@ -93,6 +98,8 @@ class UBX_PORT_Frame(Frame):
             width=6,
             state=READONLY,
             wrap=True,
+            repeatdelay=RPTDELAY,
+            repeatinterval=RPTDELAY,
             textvariable=self._bpsrate,
         )
         self._lbl_inprot = Label(self, text="Input")
@@ -195,10 +202,10 @@ class UBX_PORT_Frame(Frame):
             self._outprot_nmea.set(msg.outNMEA)
             self._outprot_rtcm3.set(msg.outRTCM3)
             self._lbl_send_command.config(image=self._img_confirmed)
-            self.__container.set_status("CFG-PRT GET message received", "green")
+            self.__container.set_status("CFG-PRT GET message received", OKCOL)
 
         elif msg.identity == "ACK-NAK":
-            self.__container.set_status("CFG-PRT POLL message rejected", "red")
+            self.__container.set_status("CFG-PRT POLL message rejected", ERRCOL)
             self._lbl_send_command.config(image=self._img_warn)
 
     def _on_select_portid(self):
