@@ -38,6 +38,7 @@ from pyubx2 import (
 )
 
 from pygpsclient.globals import (
+    ERRCOL,
     HOME,
     ICON_DELETE,
     ICON_LOAD,
@@ -48,7 +49,7 @@ from pygpsclient.globals import (
     ICON_UNDO,
 )
 from pygpsclient.helpers import set_filename
-from pygpsclient.strings import LBLCFGRECORD, READTITLE, SAVETITLE
+from pygpsclient.strings import LBLCFGRECORD, SAVETITLE
 
 STOP = 0
 PLAY = 1
@@ -106,7 +107,7 @@ class UBX_Recorder_Frame(Frame):
         Set up frame and widgets.
         """
 
-        self._lbl_recorder = Label(self, text=LBLCFGRECORD, anchor="w")
+        self._lbl_recorder = Label(self, text=LBLCFGRECORD, anchor=W)
 
         self._btn_load = Button(
             self,
@@ -204,20 +205,14 @@ class UBX_Recorder_Frame(Frame):
         Open configuration file.
         """
 
-        self._configfile = filedialog.askopenfilename(
-            parent=self.__container,
-            title=READTITLE,
-            initialdir=HOME,
-            defaultextension="ubx",
-            filetypes=(
+        return self.__app.file_handler.open_file(
+            "ubx",
+            (
                 ("ubx config files", "*.ubx"),
                 ("u-center config files", "*.txt"),
                 ("all files", "*.*"),
             ),
         )
-        if self._configfile in ((), ""):
-            return None  # User cancelled
-        return self._configfile
 
     def _on_load(self):
         """
@@ -475,7 +470,7 @@ class UBX_Recorder_Frame(Frame):
         """
 
         try:
-            cols = [("white", "red"), ("red", "white")]
+            cols = [("white", ERRCOL), (ERRCOL, "white")]
             i = 0
             while not stop.is_set():
                 i = not i
