@@ -15,16 +15,19 @@ from tkinter import Button, E, Frame, IntVar, Label, Spinbox, StringVar, W
 from PIL import Image, ImageTk
 from pyubx2 import POLL, SET, UBXMessage
 
-from .globals import (
+from pygpsclient.globals import (
     CONNECTED,
+    ERRCOL,
     ICON_CONFIRMED,
     ICON_PENDING,
     ICON_SEND,
     ICON_WARNING,
+    OKCOL,
     READONLY,
+    RPTDELAY,
     UBX_CFGRATE,
 )
-from .strings import LBLCFGRATE
+from pygpsclient.strings import LBLCFGRATE
 
 TIMEREFS = {
     0: "UTC",
@@ -73,11 +76,11 @@ class UBX_RATE_Frame(Frame):
         Set up frame and widgets.
         """
 
-        self._lbl_cfg_rate = Label(self, text=LBLCFGRATE, anchor="w")
+        self._lbl_cfg_rate = Label(self, text=LBLCFGRATE, anchor=W)
         self._lbl_ubx_measint = Label(self, text="Solution Interval (ms)")
         self._spn_ubx_measint = Spinbox(
             self,
-            values=(25, 50, 100, 200, 500, 1000, 2000, 5000, 10000),
+            values=(25, 50, 100, 200, 250, 500, 1000, 2000, 5000, 10000),
             width=6,
             state=READONLY,
             wrap=True,
@@ -91,6 +94,8 @@ class UBX_RATE_Frame(Frame):
             width=4,
             state=READONLY,
             wrap=True,
+            repeatdelay=RPTDELAY,
+            repeatinterval=RPTDELAY,
             textvariable=self._navrate,
         )
         self._lbl_ubx_timeref = Label(self, text="Time Reference")
@@ -100,6 +105,8 @@ class UBX_RATE_Frame(Frame):
             width=5,
             state=READONLY,
             wrap=True,
+            repeatdelay=RPTDELAY,
+            repeatinterval=RPTDELAY,
             textvariable=self._timeref,
         )
         self._lbl_send_command = Label(self, image=self._img_pending)
@@ -171,10 +178,10 @@ class UBX_RATE_Frame(Frame):
             self._navrate.set(msg.navRate)
             self._timeref.set(TIMEREFS[msg.timeRef])
             self._lbl_send_command.config(image=self._img_confirmed)
-            self.__container.set_status("CFG-RATE GET message received", "green")
+            self.__container.set_status("CFG-RATE GET message received", OKCOL)
 
         elif msg.identity == "ACK-NAK":
-            self.__container.set_status("CFG-RATE POLL message rejected", "red")
+            self.__container.set_status("CFG-RATE POLL message rejected", ERRCOL)
             self._lbl_send_command.config(image=self._img_warn)
 
     def _on_send_rate(self, *args, **kwargs):  # pylint: disable=unused-argument
